# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import Dependency

class VcRedist(Dependency):
    name = "vcredist"
    description = "Visual studio redistributable packages"
    default = "2005"

    install_params = {
        "2005": "/q:a",
        "2008": "/qb",
        "2010": "/passive /norestart",
        "2012": "/passive /norestart",
        "2013": "/passive /norestart",
        "2015": "/passive /norestart",
    }

    exes = [
        {
            "version": "2005",
            "arch": "x86",
            "url": "https://cuckoo.sh/vmcloak/vcredist_2005_x86.exe",
            "sha1": "7dfa98be78249921dd0eedb9a3dd809e7d215c8d"
        },
        {
            "version": "2005",
            "arch": "amd64",
            "url": "https://cuckoo.sh/vmcloak/vcredist_2005_x64.exe",
            "sha1": "756f2c773d4733e3955bf7d8f1e959a7f5634b1a"
        },
        {
            "version": "2008",
            "arch": "x86",
            "url": "https://cuckoo.sh/vmcloak/vcredist_2008_x86.exe",
            "sha1": "6939100e397cef26ec22e95e53fcd9fc979b7bc9"
        },
        {
            "version": "2008",
            "arch": "amd64",
            "url": "https://cuckoo.sh/vmcloak/vcredist_2008_x64.exe",
            "sha1": "13674c43652b941dafd2049989afce63cb7c517b"
        },
        {
            "version": "2010",
            "arch": "x86",
            "url": "https://cuckoo.sh/vmcloak/vcredist_2010_x86.exe",
            "sha1": "b84b83a8a6741a17bfb5f3578b983c1de512589d"
        },
        {
            "version": "2010",
            "arch": "amd64",
            "url": "https://cuckoo.sh/vmcloak/vcredist_2010_x64.exe",
            "sha1": "027d0c2749ec5eb21b031f46aee14c905206f482"
        },
        {
            "version": "2012",
            "arch": "x86",
            "url": "https://cuckoo.sh/vmcloak/vcredist_2012_x86.exe",
            "sha1": "7d6f654c16f9ce534bb2c4b1669d7dc039c433c9"
        },
        {
            "version": "2012",
            "arch": "amd64",
            "url": "https://cuckoo.sh/vmcloak/vcredist_2012_x64.exe",
            "sha1": "c4ac45564e801e1bfd87936cac8a76c5754cdbd4"
        },
        {
            "version": "2013",
            "arch": "x86",
            "url": "https://cuckoo.sh/vmcloak/vcredist_2013_x86.exe",
            "sha1": "df7f0a73bfa077e483e51bfb97f5e2eceedfb6a3"
        },
        {
            "version": "2013",
            "arch": "amd64",
            "url": "https://cuckoo.sh/vmcloak/vcredist_2013_x64.exe",
            "sha1": "8bf41ba9eef02d30635a10433817dbb6886da5a2"
        },
        {
            "version": "2015",
            "arch": "x86",
            "url": "https://cuckoo.sh/vmcloak/vcredist_2015_x86.exe",
            "sha1": "bfb74e498c44d3a103ca3aa2831763fb417134d1"
        },
        {
            "version": "2015",
            "arch": "amd64",
            "url": "https://cuckoo.sh/vmcloak/vcredist_2015_x64.exe",
            "sha1": "3155cb0f146b927fcc30647c1a904cd162548c8c"
        },
    ]

    def run(self):
        # Get the installation parameters required to start
        # an unattended install
        param = self.install_params[self.version]

        self.upload_dependency("C:\\vcredist.exe")
        self.a.execute("C:\\vcredist.exe %s" % param)
        self.a.remove("C:\\vcredist.exe")



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.
# Chrome Dependency submitted by Jason Lewis.
# https://dl.google.com/dl/chrome/install/googlechromestandaloneenterprise.msi

from vmcloak.abstract import Dependency

class Chrome(Dependency):
    name = "chrome"
    exes = [
        {
            "url": "https://cuckoo.sh/vmcloak/googlechromestandaloneenterprise.msi",
            "sha1": "a0ade494dda8911eeb68c9294c2dd0e3229d8f02",
        },
    ]

    def run(self):
        self.upload_dependency("C:\\googlechromestandaloneenterprise.msi")
        self.a.execute("Msiexec /q /I C:\\googlechromestandaloneenterprise.msi")
        self.a.remove("C:\\googlechromestandaloneenterprise.msi")



# Copyright (C) 2014-2017 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import Dependency

config = """
INSTALL_SILENT=Enable
AUTO_UPDATE=Disable
EULA=Disable
SPONSORS=Disable
WEB_JAVA_SECURITY_LEVEL=H
WEB_ANALYTICS=Disable
"""

class Java(Dependency):
    name = "java"
    default = "7"
    recommended = True
    exes = [
        # http://www.oracle.com/technetwork/java/javase/downloads/java-archive-downloads-javase7-521261.html
        {
            "version": "7",
            "url": "https://cuckoo.sh/vmcloak/jdk-7-windows-i586.exe",
            "sha1": "2546a78b6138466b3e23e25b5ca59f1c89c22d03",
        },
        {
            "version": "7u1",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u1-windows-i586.exe",
            "sha1": "ed434b8bc132a5bfda031428b26daf7b8331ecb9",
        },
        {
            "version": "7u2",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u2-windows-i586.exe",
            "sha1": "a36ae80b80dd1c9c5c466b3eb2451cd649613cfb",
        },
        {
            "version": "7u3",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u3-windows-i586.exe",
            "sha1": "fe9dc13c0a6424158dc0f13a6246a53973fb5369",
        },
        {
            "version": "7u4",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u4-windows-i586.exe",
            "sha1": "a2e927632b2106f5efefc906ed9070d8c0bf660f",
        },
        {
            "version": "7u5",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u5-windows-i586.exe",
            "sha1": "88c2fc5e5e61e7f709370c01abb138c65009307b",
        },
        {
            "version": "7u6",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u6-windows-i586.exe",
            "sha1": "09f3a1d0fe7fabd4cfdc1c23d1ed16016d064d01",
        },
        {
            "version": "7u7",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u7-windows-i586.exe",
            "sha1": "58e4bdd12225379284542b161e49d8eaea4e00c2",
        },
        {
            "version": "7u9",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u9-windows-i586.exe",
            "sha1": "11a256bd791033527580c6ac8700f3a72f7f4bcf",
        },
        {
            "version": "7u10",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u10-windows-i586.exe",
            "sha1": "f57bfa38a05433d902582fab9d08f530d7c7749b",
        },
        {
            "version": "7u11",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u11-windows-i586.exe",
            "sha1": "a482e48e151cff76dcc1479b9efc367da8fb66a7",
        },
        {
            "version": "7u13",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u13-windows-i586.exe",
            "sha1": "bd6848138385510b32897a5b04c94aa4cf2b4fca",
        },
        {
            "version": "7u15",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u15-windows-i586.exe",
            "sha1": "f52453c6fd665b89629e639abdb41492eff9a9e3",
        },
        {
            "version": "7u17",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u17-windows-i586.exe",
            "sha1": "1f462dea65c74dd9fdf094d852e438a0e6a036bc",
        },
        {
            "version": "7u21",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u21-windows-i586.exe",
            "sha1": "f677efa8309e99fe3a47ea09295b977af01f2142",
        },
        {
            "version": "7u25",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u25-windows-i586.exe",
            "sha1": "5eeb8869f9abcb8d575a7f75a6f85550edf680f5",
        },
        {
            "version": "7u40",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u40-windows-i586.exe",
            "sha1": "b611fb48bb5071b54ef45633cd796a27d5cd0ffd",
        },
        {
            "version": "7u45",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u45-windows-i586.exe",
            "sha1": "cfd7e00fa0f6b3eef32832dd7487c6f56e7f55b8",
        },
        {
            "version": "7u51",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u51-windows-i586.exe",
            "sha1": "439435a1b40053761e3a555e97befb4573c303e5",
        },
        {
            "version": "7u55",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u55-windows-i586.exe",
            "sha1": "bb244a96e58724415380877230d2f6b466e9e581",
        },
        {
            "version": "7u60",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u60-windows-i586.exe",
            "sha1": "8f9185b1fb80dee64e511e222c1a9742eff7837f",
        },
        {
            "version": "7u65",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u65-windows-i586.exe",
            "sha1": "9c52a8185b9931b8ae935adb63c8272cf6d9e9ba",
        },
        {
            "version": "7u67",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u67-windows-i586.exe",
            "sha1": "dff04608d4c045cdd66dffe726aed27b22939c9e",
        },
        {
            "version": "7u71",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u71-windows-i586.exe",
            "sha1": "8ca5c5ad43148dfc0e5640db114e317f1bbd6a25",
        },
        {
            "version": "7u72",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u72-windows-i586.exe",
            "sha1": "57f7dff98bdfbe064af159bbd1d8753cad714f68",
        },
        {
            "version": "7u75",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u75-windows-i586.exe",
            "sha1": "700e56c9b57f5349d4fe9ba28878973059dc68fa",
        },
        {
            "version": "7u76",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u76-windows-i586.exe",
            "sha1": "0469ba6302aa3dc03e39075451aef1c60e5e4114",
        },
        {
            "version": "7u79",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u79-windows-i586.exe",
            "sha1": "319306c148c97f404c00e5562b11f5f4ea5fd6e5",
        },
        {
            "version": "7u80",
            "url": "https://cuckoo.sh/vmcloak/jdk-7u80-windows-i586.exe",
            "sha1": "aebbc0b02c16e7169b0577962fa91c613f8a7a45",
        },
        # http://www.oracle.com/technetwork/java/javase/downloads/java-archive-javase8-2177648.html
        {
            "version": "8",
            "url": "https://cuckoo.sh/vmcloak/jdk-8-windows-i586.exe",
            "sha1": "09a05b1afad97ffa35a47d571752c3e804c200c7",
        },
        {
            "version": "8u5",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u5-windows-i586.exe",
            "sha1": "81660732a53e08651c633d99b0e6042cbbaf616d",
        },
        {
            "version": "8u11",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u11-windows-i586.exe",
            "sha1": "757103707b16e6a79ebd4d134613e483007a0c7a",
        },
        {
            "version": "8u20",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u20-windows-i586.exe",
            "sha1": "30df3349f710e6b54adccadadc1e1f939ab2f6da",
        },
        {
            "version": "8u25",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u25-windows-i586.exe",
            "sha1": "79b4b68aea5ef6448c39c2ee3103722db6548ff0",
        },
        {
            "version": "8u31",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u31-windows-i586.exe",
            "sha1": "5b8a1f8d11ecbcd46ed3389498ef67a4f699133b",
        },
        {
            "version": "8u40",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u40-windows-i586.exe",
            "sha1": "ff9f4d62dffa0a81abbc0e5e151586301ddf4884",
        },
        {
            "version": "8u45",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u45-windows-i586.exe",
            "sha1": "8e839fe0e30a56784566017f6acdb0fbe213c8bc",
        },
        {
            "version": "8u51",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u51-windows-i586.exe",
            "sha1": "0aaee8ff5f62fdcb3685d513be471c49824d7e04",
        },
        {
            "version": "8u60",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u60-windows-i586.exe",
            "sha1": "47b36bc0fdc44029f82a50346fbb85b8f7803d7f",
        },
        {
            "version": "8u65",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u65-windows-i586.exe",
            "sha1": "66bdacc1252f309f157fd0786d2e148dbb394629",
        },
        {
            "version": "8u66",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u66-windows-i586.exe",
            "sha1": "0013f600723a1a16aa97f7c3fbe1c27fd7674cbe",
        },
        {
            "version": "8u71",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u71-windows-i586.exe",
            "sha1": "c6726fb46cb40b42b4b545502ee87172b7d290f5",
        },
        {
            "version": "8u72",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u72-windows-i586.exe",
            "sha1": "d1b6e793c21f1bec935f647ec49a12bc54109ace",
        },
        {
            "version": "8u73",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u73-windows-i586.exe",
            "sha1": "f56e21ece567f42fce5a38961bd81288dd2956c0",
        },
        {
            "version": "8u74",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u74-windows-i586.exe",
            "sha1": "8fa2c7f22b9176d0201d40dc21c29bc7002f5251",
        },
        {
            "version": "8u77",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u77-windows-i586.exe",
            "sha1": "1560add14dde3e4c5bac020116f5bc06d49be567",
        },
        {
            "version": "8u91",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u91-windows-i586.exe",
            "sha1": "5374b68f6cca15345fd7d8de0b352cd37804068d",
        },
        {
            "version": "8u92",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u92-windows-i586.exe",
            "sha1": "b89aa89d66ea1783628f62487a137c993af7ca8b",
        },
        {
            "version": "8u101",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u101-windows-i586.exe",
            "sha1": "2d2d56f5774cc2f15d9e54bebc9a868913e606b7",
        },
        {
            "version": "8u102",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u102-windows-i586.exe",
            "sha1": "3acf0fca1d5bf56f8a2ce577d055bfd0dd1773f9",
        },
        {
            "version": "8u121",
            "url": "https://cuckoo.sh/vmcloak/jdk-8u121-windows-i586.exe",
            "sha1": "e71fc3eb9f895eba5c2836b05d627884edd0157a",
        },
    ]

    def run(self):
        self.upload_dependency("C:\\java.exe")

        if self.version.startswith("7"):
            self.a.execute("C:\\java.exe /s WEB_JAVA=1 WEB_JAVA_SECURITY_LEVEL=M SPONSORS=0", async=True)
        else:
            self.a.upload("C:\\config.cfg", config)
            self.a.execute("C:\\java.exe INSTALLCFG=C:\\config.cfg", async=True)

        # Wait until java.exe & javaw.exe are no longer running.
        self.wait_process_exit("java.exe")
        self.wait_process_exit("javaw.exe")

        self.a.remove("C:\\java.exe")

        if not self.version.startswith("7"):
            self.a.remove("C:\\config.cfg")

        if self.i.osversion == "winxp" or self.i.osversion == "win7x86":
            self.a.execute("reg add \"HKEY_LOCAL_MACHINE\\SOFTWARE\\JavaSoft\\Java Update\\Policy\" /v EnableJavaUpdate /t REG_DWORD /d 0 /f")

        if self.i.osversion == "win7x64":
            self.a.execute("reg add \"HKEY_LOCAL_MACHINE\\SOFTWARE\\Wow6432Node\\JavaSoft\\Java Update\\Policy\" /v EnableJavaUpdate /t REG_DWORD /d 0 /f")

class Java7(Java, Dependency):
    """Backwards compatibility."""
    name = "java7"
    recommended = False



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import random
import string

def random_string(minimum, maximum=None):
    if maximum is None:
        maximum = minimum

    count = random.randint(minimum, maximum)
    return "".join(random.choice(string.ascii_letters) for x in xrange(count))

def random_mac():
    """Generates a random MAC address."""
    values = [random.randint(0, 15) for _ in xrange(12)]

    # At least for VirtualBox there's a limitation for the second character,
    # as outlined in the following thread. Thus we handle this.
    # https://forums.virtualbox.org/viewtopic.php?p=85316
    values[1] = int(random.choice("02468ace"), 16)

    return "%x%x:%x%x:%x%x:%x%x:%x%x:%x%x" % tuple(values)

def random_serial(length=None):
    if length is None:
        length = random.randint(8, 20)

    return "".join(random.choice(string.uppercase + string.digits)
                   for _ in xrange(length))

def random_uuid():
    value = random_serial(32)
    return "-".join((value[:8], value[8:12], value[12:16],
                     value[16:20], value[20:32]))



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import sys

from ctypes import c_char, c_ushort, c_uint, c_char_p
from ctypes import windll, Structure, POINTER, sizeof

# http://blogs.technet.com/b/heyscriptingguy/archive/2010/07/07/hey-scripting-guy-how-can-i-change-my-desktop-monitor-resolution-via-windows-powershell.aspx
# http://msdn.microsoft.com/en-us/library/windows/desktop/dd183565(v=vs.85).aspx
class _DevMode(Structure):
    _fields_ = [
        ("dmDeviceName", c_char * 32),
        ("unused1", c_ushort * 2),
        ("dmSize", c_ushort),
        ("unused2", c_ushort),
        ("unused3", c_uint * 8),
        ("dmFormName", c_char * 32),
        ("dmLogPixels", c_ushort),
        ("dmBitsPerPel", c_ushort),
        ("dmPelsWidth", c_uint),
        ("dmPelsHeight", c_uint),
        ("unused2", c_uint * 10),
    ]

EnumDisplaySettings = windll.user32.EnumDisplaySettingsA
EnumDisplaySettings.argtypes = c_char_p, c_uint, POINTER(_DevMode)

ChangeDisplaySettings = windll.user32.ChangeDisplaySettingsA
ChangeDisplaySettings.argtypes = POINTER(_DevMode), c_uint

ENUM_CURRENT_SETTINGS = -1
CDS_UPDATEREGISTRY = 1
DISP_CHANGE_SUCCESSFUL = 0

if __name__ == "__main__":
    if len(sys.argv) != 3:
        print "Usage: %s <width> <height>" % sys.argv[0]
        exit(1)

    dm = _DevMode()
    dm.dmSize = sizeof(dm)
    if not EnumDisplaySettings(None, ENUM_CURRENT_SETTINGS, dm):
        print "Error enumerating display settings.."
        exit(1)

    dm.dmPelsWidth = int(sys.argv[1])
    dm.dmPelsHeight = int(sys.argv[2])

    ret = ChangeDisplaySettings(dm, CDS_UPDATEREGISTRY)
    if ret != DISP_CHANGE_SUCCESSFUL:
        print "Error changing display settings.."
        exit(1)



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import json
import logging
import os

from vmcloak.constants import VMCLOAK_ROOT

log = logging.getLogger(__name__)
HWCONF_PATH = os.path.join(VMCLOAK_ROOT, "data", "hwconf")

def load_hwconf(profile, dirpath=HWCONF_PATH):
    ret = {}

    if profile is not None:
        files = ["%s.json" % profile]
    else:
        # Load profiles that ship with VMCloak.
        files = os.listdir(dirpath)

        # Load local profiles.
        local_hwconf = os.path.join(os.getenv("HOME"),
                                    ".config", "vmcloak", "hwconf")

        if os.path.exists(local_hwconf):
            for fname in os.listdir(local_hwconf):
                files.append(os.path.join(local_hwconf, fname))

    for fname in files:
        if not fname.endswith(".json"):
            continue

        conf = json.load(open(os.path.join(dirpath, fname), "rb"))
        for key, value in conf.items():
            if key not in ret:
                ret[key] = []

            if isinstance(value, list):
                ret[key].extend(value)
            else:
                ret[key].append(value)

    return ret



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import WindowsAutounattended

class Windows7(WindowsAutounattended):
    name = "win7"
    service_pack = 2
    interface = "Local Area Connection"

    # List of preferences when multiple Windows 7 types are available.
    preference = "professional", "homepremium", "ultimate", "homebasic"

    dummy_serial_key = "33PXH-7Y6KF-2VJC9-XBBR8-HVTHH"

class Windows7x64(Windows7):
    arch = "amd64"
    mount = "/mnt/win7x64", "/mnt/win7"

class Windows7x86(Windows7):
    arch = "x86"
    mount = "/mnt/win7x86", "/mnt/win7"



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.
# Chrome Dependency submitted by Jason Lewis.

from vmcloak.abstract import Dependency

class RemoveTooltips(Dependency):
    name = "removetooltips"
    description = "Removes balloon tooltips for new users"
    exes = [
        {
            "url": "https://cuckoo.sh/vmcloak/MicrosoftFixit50048.msi",
            "sha1": "0de68031b1c1d17bf6851b13f2d083ee61b6b533",
        },
    ]

    def run(self):
        self.upload_dependency("C:\\MicrosoftFixit50048.msi")
        self.a.execute("msiexec /i C:\\MicrosoftFixit50048.msi /passive /norestart")
        self.a.remove("C:\\MicrosoftFixit50048.msi")



# Copyright (C) 2014-2015 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.
# 
# The Windows Imaging Component (WIC) provides WIC-enabled 
# applications to display and edit any image format for which
# a WIC-compliant CODEC is installed, and also to read and 
# write metadata in image files. 

from vmcloak.abstract import Dependency

class WIC(Dependency):
    name = "wic"
    exes = [
        {
            "url": "https://cuckoo.sh/vmcloak/wic_x86_enu.exe",
            "sha1": "53c18652ac2f8a51303deb48a1b7abbdb1db427f",
        },
    ]

    def run(self):
        self.upload_dependency("C:\\wic.exe")
        self.a.execute("C:\\wic.exe /passive /norestart")
        self.a.remove("C:\\wic.exe")



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import time

from vmcloak.abstract import Dependency

class Cuteftp(Dependency):
    name = "cuteftp"
    default = "9.0.5"
    exes = [
        {
            "version": "9.0.5",
            "url": "https://cuckoo.sh/vmcloak/cuteftp.exe",
            "sha1": "1d8497b3f31f76168eb2573efe60dcefb3422e1d",
        },
    ]

    def run(self):
        self.upload_dependency("C:\\%s" % self.filename)
        self.a.execute("C:\\%s /S" % self.filename, async=True)

        time.sleep(1)
        self.a.click("InstallShield Wizard", "&Next >")
        self.a.click("InstallShield Wizard", "&Yes")
        self.a.click("InstallShield Wizard", "&Next >")
        time.sleep(1)
        self.a.click("InstallShield Wizard", "Finish")

        self.wait_process_exit("cuteftp.exe")

        self.a.remove("C:\\%s" % self.filename)



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import Dependency

class Flash(Dependency):
    name = "flash"
    default = "11.7.700.169"
    recommended = True
    exes = [
        {
            "version": "11.4.402.287",
            "url": "https://cuckoo.sh/vmcloak/flashplayer11_4r402_287_winax.msi",
            "sha1": "99fb61ed221df9125698e78d659ee1fc93b97c60",
        },
        {
            "version": "11.6.602.168",
            "url": "https://cuckoo.sh/vmcloak/flashplayer11_6r602_168_winax.msi",
            "sha1": "906f920563403ace6dee9b0cff982ea02d4b5c06",
        },
        {
            "version": "11.7.700.169",
            "url": "https://cuckoo.sh/vmcloak/flashplayer11_7r700_169_winax.msi",
            "sha1": "790b09c63c44f0eafd7151096bf58964857d3b17",
        },
        {
            "version": "11.8.800.94",
            "url": "https://cuckoo.sh/vmcloak/flashplayer11_8r800_94_winax.msi",
            "sha1": "f7a152aa3af4a7bbef4f87ab5e05d24824ddf439",
        },
        {
            "version": "11.8.800.174",
            "url": "https://cuckoo.sh/vmcloak/flashplayer11_8r800_174_winax.msi",
            "sha1": "f3a466b33e11a72f3bef4ecd40ef63c19f97a077",
        },
        {
            "version": "11.9.900.117",
            "url": "https://cuckoo.sh/vmcloak/flashplayer11_9r900_117_winax.msi",
            "sha1": "982cf5626174e42814a7bb27ff62378b230dc201",
        },
        {
            "version": "11.9.900.170",
            "url": "https://cuckoo.sh/vmcloak/flashplayer11_9r900_170_winax.msi",
            "sha1": "877c7e9a6de692bdef95a4f3cc22b9fb385db92e",
        },
        {
            "version": "12.0.0.38",
            "url": "https://cuckoo.sh/vmcloak/flashplayer12_0r0_38_winax.msi",
            "sha1": "e44908c9a813876c1a332a174b2e3e9dff47a4ff",
        },
        {
            "version": "13.0.0.182",
            "url": "https://cuckoo.sh/vmcloak/flashplayer13_0r0_182_winax.msi",
            "sha1": "fbe7a2698da29284be8366adaf8765e9990fd6e0",
        },
        {
            "version": "13.0.0.214",
            "url": "https://cuckoo.sh/vmcloak/flashplayer13_0r0_214_winax.msi",
            "sha1": "c79bfbb23b497cec805164df9f038644d74476aa",
        },
        {
            "version": "14.0.0.125",
            "url": "https://cuckoo.sh/vmcloak/flashplayer14_0r0_125_winax.msi",
            "sha1": "f6bd2b5e91195182898828cc1235fd27d2aa8d55",
        },
        {
            "version": "15.0.0.167",
            "url": "https://cuckoo.sh/vmcloak/flashplayer15_0r0_167_winax.msi",
            "sha1": "79f04ea94d033c07e7cc889751769e98f99d23fb",
        },
        {
            "version": "15.0.0.189",
            "url": "https://cuckoo.sh/vmcloak/flashplayer15_0r0_189_winax.msi",
            "sha1": "8bfd539853e0db0a58efdb8c5f5c021f1fcb9f8d",
        },
        {
            "version": "16.0.0.235",
            "url": "https://cuckoo.sh/vmcloak/flashplayer16_0r0_235_winax.msi",
            "sha1": "ad3da00825193ecee48440203b6051310f4de3b7",
        },
        {
            "version": "18.0.0.194",
            "url": "https://cuckoo.sh/vmcloak/flashplayer18_0r0_194_winax.msi",
            "sha1": "2611f2d81ea8ef747f079d797099da89648566ed",
        },
        {
            "version": "18.0.0.203",
            "url": "https://cuckoo.sh/vmcloak/flashplayer18_0r0_203_winax.msi",
            "sha1": "98ab0531be9f2b49adeb56c621678c17455e6f66",
        },
        {
            "version": "18.0.0.209",
            "url": "https://cuckoo.sh/vmcloak/flashplayer18_0r0_209_winax.msi",
            "sha1": "a209b6a320965cfbd75455ee79ab28e03bd5371c",
        },
        {
            "version": "19.0.0.207",
            "url": "https://cuckoo.sh/vmcloak/flashplayer19_0r0_207_winax.msi",
            "sha1": "9a901f938add920ed846ae18187176cbdb40ecfb",
        },
        {
            "version": "19.0.0.245",
            "url": "https://cuckoo.sh/vmcloak/flashplayer19_0r0_245_winax.msi",
            "sha1": "1efef336190b021f6df3325f958f1698a91cce8c",
        },
        {
            "version": "20.0.0.228",
            "url": "https://cuckoo.sh/vmcloak/flashplayer20_0d0_228_winax.msi",
            "sha1": "fa86f107111fc7def35742097bf0aa29c82d7638",
        },
    ]

    def run(self):
        self.upload_dependency("C:\\%s" % self.filename)
        self.a.execute("msiexec /i C:\\%s /passive" % self.filename)
        self.a.remove("C:\\%s" % self.filename)



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import Dependency

class Win7sp(Dependency):
    name = "win7sp"
    exes = [
        {
            "version": "1",
            "target": "win7x64",
            "url": "https://cuckoo.sh/vmcloak/windows6.1-KB976932-X64.exe",
            "sha1": "74865ef2562006e51d7f9333b4a8d45b7a749dab",
        },
        {
            "version": "1",
            "target": "win7x86",
            "url": "https://cuckoo.sh/vmcloak/windows6.1-KB976932-X86.exe",
            "sha1": "c3516bc5c9e69fee6d9ac4f981f5b95977a8a2fa",
        },
    ]

    def run(self):
        self.upload_dependency("C:\\setup.exe")
        self.a.execute("C:\\setup.exe /quiet /norestart")
        self.a.remove("C:\\setup.exe")



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import logging
import os.path
import random
import re
import shutil
import subprocess
import tempfile
import time

from vmcloak.conf import load_hwconf
from vmcloak.constants import VMCLOAK_ROOT
from vmcloak.exceptions import DependencyError
from vmcloak.misc import copytreelower, copytreeinto, sha1_file, ini_read
from vmcloak.paths import get_path
from vmcloak.rand import random_serial, random_uuid, random_string
from vmcloak.repository import deps_path
from vmcloak.verify import valid_serial_key

log = logging.getLogger(__name__)

GENISOIMAGE_WARNINGS = [
    "Warning: creating filesystem that does not conform to ISO-9660.",
    "Warning: creating filesystem that does not conform to ISO-9660."
    " Warning: creating filesystem with (nonstandard) Joliet extensions but"
    " without (standard) Rock Ridge extensions. It is highly recommended to"
    " add Rock Ridge",
]


class Machinery(object):
    FIELDS = {}
    vm_dir_required = True
    data_dir_required = True

    def __init__(self, name):
        self.name = name
        self.network_idx = 0

    def vminfo(self, element=None):
        """Returns a dictionary with all available information for the
        Virtual Machine."""
        raise

    def create_vm(self):
        """Create a new Virtual Machine."""
        raise

    def delete_vm(self):
        """Delete an existing Virtual Machine and its associated files."""
        raise

    def ramsize(self, ramsize):
        """Modify the amount of RAM available for this Virtual Machine."""
        raise

    def vramsize(self, vramsize):
        """Modify the amount of Video memory available for this Virtual
        Machine."""
        raise

    def os_type(self, osversion):
        """Set the OS type."""
        raise

    def create_hd(self, fsize):
        """Create a harddisk."""
        raise

    def immutable_hd(self):
        """Make a harddisk immutable or normal."""
        raise

    def remove_hd(self):
        """Remove a harddisk."""
        raise

    def clone_hd(self, hdd_inpath, hdd_outpath):
        """Clone a harddisk."""
        raise

    def cpus(self, count):
        """Set the number of CPUs to assign to this Virtual Machine."""
        raise

    def attach_iso(self, iso):
        """Attach a ISO file as DVDRom drive."""
        raise

    def detach_iso(self):
        """Detach the ISO file in the DVDRom drive."""
        raise

    def set_field(self, key, value):
        """Set a specific field of a Virtual Machine."""
        raise

    def modify_mac(self, mac=None):
        """Modify the MAC address of a Virtual Machine."""
        raise

    def network_index(self):
        """Get the index for the next network interface."""
        ret = self.network_idx
        self.network_idx += 1
        return ret

    def hostonly(self, macaddr=None, index=1):
        """Configure a hostonly adapter for the Virtual Machine."""
        raise

    def nat(self, macaddr=None, index=1):
        """Configure NAT for the Virtual Machine."""
        raise

    def hwvirt(self, enable=True):
        """Enable or disable the usage of Hardware Virtualization."""
        raise

    def start_vm(self, visible=False):
        """Start the associated Virtual Machine."""
        raise

    def snapshot(self, label):
        """Take a snapshot of the associated Virtual Machine."""
        raise

    def stopvm(self):
        """Stop the associated Virtual Machine."""
        raise

    def list_settings(self):
        """List all settings of a Virtual Machine."""
        raise

    def init_vm(self, profile):
        """Initialize fields as specified by `FIELDS`."""
        hwconf = load_hwconf(profile=profile)

        def _init_vm(path, fields):
            for key, value in fields.items():
                key = path + "/" + key
                if isinstance(value, dict):
                    _init_vm(key, value)
                else:
                    if isinstance(value, tuple):
                        k, v = value
                        if k not in hwconf or not hwconf[k]:
                            value = "To be filled by O.E.M."
                        else:
                            if k not in config:
                                config[k] = random.choice(hwconf[k])

                            value = config[k][v]

                            # Some values have to be generated randomly.
                            if value is not None:
                                if value.startswith("<SERIAL>"):
                                    length = int(value.split()[-1])
                                    value = random_serial(length)
                                elif value.startswith("<UUID>"):
                                    value = random_uuid()

                    if value is None:
                        value = "To be filled by O.E.M."

                    log.debug("Setting %r to %r.", key, value)
                    ret = self.set_field(key, value)
                    if ret:
                        log.debug(ret)

        config = {}
        _init_vm("", self.FIELDS)


class OperatingSystem(object):
    # Short name for this OS.
    name = None

    # Service Pack that is likely being used.
    service_pack = None

    # Architecture this OS is aimed at (x86 or amd64).
    arch = None

    # Default directory where the original ISO is mounted.
    mount = None

    # The Network Interface Card Type.
    nictype = None

    # Directory where to store the vmcloak bootstrap files.
    osdir = None

    # Interface name in Windows
    interface = None

    # Additional arguments for genisoimage.
    genisoargs = []

    def __init__(self):
        self.data_path = os.path.join(VMCLOAK_ROOT, "data")
        self.path = os.path.join(self.data_path, self.name)
        self.serial_key = None

        if self.name is None:
            raise Exception("Name has to be provided for OS handler")

        if self.osdir is None:
            raise Exception("OSDir has to be provided for OS handler")

    def configure(self, tempdir, product):
        """Configure the setup with settings provided by the user."""
        self.tempdir = tempdir
        self.product = product

    def isofiles(self, outdir, tmp_dir=None):
        """Abstract method for writing additional files to the newly created
        ISO file."""

    def set_serial_key(self, serial_key):
        """Abstract method for checking a serial key if provided and otherwise
        use a default serial key if available."""

    def pickmount(self, isomount=None):
        """Picks the first available mounted directory."""
        mounts = [isomount]

        if isinstance(self.mount, basestring):
            mounts.append(self.mount)
        else:
            mounts.extend(self.mount)

        for mount in mounts:
            if mount and os.path.isdir(mount) and os.listdir(mount):
                return mount

    def buildiso(self, mount, newiso, bootstrap, tmp_dir=None):
        """Builds an ISO file containing all our modifications."""
        outdir = tempfile.mkdtemp(dir=tmp_dir)

        # Copy all files to our temporary directory as mounted iso files are
        # read-only and we need lowercase (aka case-insensitive) filepaths.
        copytreelower(mount, outdir)

        # Copy the boot image.
        shutil.copy(os.path.join(self.path, "boot.img"), outdir)

        # Allow the OS handler to write additional files.
        self.isofiles(outdir, tmp_dir)

        os.makedirs(os.path.join(outdir, self.osdir, "vmcloak"))

        data_bootstrap = os.path.join(self.data_path, "bootstrap")
        for fname in os.listdir(data_bootstrap):
            shutil.copy(os.path.join(data_bootstrap, fname),
                        os.path.join(outdir, self.osdir, "vmcloak", fname))

        copytreeinto(bootstrap, os.path.join(outdir, self.osdir))

        isocreate = get_path("genisoimage")
        if not isocreate:
            log.error("Either genisoimage or mkisofs is required!")
            shutil.rmtree(outdir)
            return False

        args = [
            isocreate, "-quiet", "-b", "boot.img", "-o", newiso,
        ] + self.genisoargs + [outdir]

        log.debug("Executing genisoimage: %s", " ".join(args))
        p = subprocess.Popen(
            args, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        out, err = p.communicate()
        warning = re.sub("[\\s]+", " ", err).strip()
        if p.wait() or out or warning not in GENISOIMAGE_WARNINGS:
            log.error(
                "Error creating ISO file (err=%d): %s %s",
                p.wait(), out, err
            )
            shutil.rmtree(outdir)
            return False

        shutil.rmtree(outdir)
        return True

class WindowsAutounattended(OperatingSystem):
    """Abstract wrapper around Windows-based Operating Systems that use the
    autounattend.xml file for automated installation, i.e., Windows 7+."""

    nictype = "82540EM"
    osdir = os.path.join("sources", "$oem$", "$1")
    dummy_serial_key = None
    genisoargs = [
        "-no-emul-boot", "-iso-level", "2", "-udf", "-J", "-l", "-D", "-N",
        "-joliet-long", "-relaxed-filenames",
    ]

    def _autounattend_xml(self, product):
        values = {
            "PRODUCTKEY": self.serial_key,
            "COMPUTERNAME": random_string(8, 14),
            "USERNAME": random_string(8, 12),
            "PASSWORD": random_string(8, 16),
            "PRODUCT": product.upper(),
            "ARCH": self.arch,
            "INTERFACE": self.interface,
        }

        buf = open(os.path.join(self.path, "autounattend.xml"), "rb").read()
        for key, value in values.items():
            buf = buf.replace("@%s@" % key, value)

        return buf

    def isofiles(self, outdir, tmp_dir=None):
        products = []

        product_ini = os.path.join(outdir, "sources", "product.ini")
        mode, conf = ini_read(product_ini)

        for line in conf.get("BuildInfo", []):
            if "=" not in line:
                continue

            key, value = line.split("=", 1)
            if key != "staged":
                continue

            for product in value.split(","):
                products.append(product.lower())

            break

        for preference in self.preference:
            if preference in products:
                product = preference
                break
        else:
            if products:
                product = products[0]
            else:
                product = self.preference[0]

        if self.product and self.product.lower() not in self.preference:
            log.error(
                "The product version of %s that was specified on the "
                "command-line is not known by us, ignoring it.", self.name
            )
            self.product = None

        with open(os.path.join(outdir, "autounattend.xml"), "wb") as f:
            f.write(self._autounattend_xml(self.product or product))

    def set_serial_key(self, serial_key):
        if serial_key and not valid_serial_key(serial_key):
            log.error("The provided serial key has an incorrect encoding.")
            log.info("Example encoding, AAAAA-BBBBB-CCCCC-DDDDD-EEEEE.")
            return False

        # https://technet.microsoft.com/en-us/library/jj612867.aspx
        self.serial_key = serial_key or self.dummy_serial_key
        return True

class Dependency(object):
    """Dependency instance. Each software has its own dependency class which
    informs VMCloak on how to install that particular piece of software."""
    name = None
    default = None
    recommended = False
    depends = None
    exes = []

    def __init__(self, h=None, m=None, a=None, i=None,
                 version=None, settings={}):
        self.h = h
        self.m = m
        self.a = a
        self.i = i
        self.version = version or self.default
        self.settings = settings
        self.exe = None
        self.filename = None

        self.init()

        # Emit settings directly into the instance. Should be fine.
        for target, value in settings.items():
            package, key = target.split(".", 1)
            if package == self.name:
                setattr(self, key, value)

        # Locate the matching installer.
        for exe in self.exes:
            if "target" in exe and exe["target"] != i.osversion:
                continue

            if "version" in exe and exe["version"] != self.version:
                continue

            if "arch" in exe and exe["arch"] != h.arch:
                continue

            self.exe = exe
            break
        else:
            if self.exes:
                log.error("Could not find the correct installer!")
                raise DependencyError

        # Download the dependency (if there is any to download).
        if self.exe:
            self.filename = os.path.basename(self.exe["url"])
            self.download()

        if self.check() is False:
            raise DependencyError

    def download(self):
        url, sha1 = self.exe["url"], self.exe["sha1"]
        self.filepath = os.path.join(deps_path, os.path.basename(url))
        if not os.path.exists(self.filepath) or \
                sha1_file(self.filepath) != sha1:
            subprocess.call(["wget", "-O", self.filepath, url])

        if sha1_file(self.filepath) != sha1:
            log.error("The checksum of %s doesn't match!", self.name)
            raise DependencyError

    def init(self):
        pass

    def check(self):
        pass

    def run(self):
        raise NotImplementedError

    def disable_autorun(self):
        """Disables AutoRun under Windows XP and Windows 7."""
        if self.h.name == "winxp":
            self.a.execute("reg add HKEY_LOCAL_MACHINE\\Software\\Microsoft\\"
                           "Windows\\CurrentVersion\\Policies\\Explorer "
                           "/v NoDriveTypeAutoRun /t REG_DWORD /d 177 /f")

        if self.h.name == "win7":
            self.a.execute("reg add HKEY_LOCAL_MACHINE\\Software\\Microsoft\\"
                           "Windows\\CurrentVersion\\Policies\\Explorer "
                           "/v NoDriveTypeAutoRun /t REG_DWORD /d 255 /f")

    def upload_dependency(self, filepath):
        """Upload this dependency to the specified filepath."""
        self.a.upload(filepath, open(self.filepath, "rb"))

    def wait_process_appear(self, process_name):
        """Wait for a process to appear."""
        while True:
            time.sleep(1)

            for line in self.a.execute("tasklist").json()["stdout"].split("\n"):
                if line.lower().startswith(process_name.lower()):
                    return

    def wait_process_exit(self, process_name):
        """Wait for a process to exit."""
        while True:
            time.sleep(1)

            for line in self.a.execute("tasklist").json()["stdout"].split("\n"):
                if line.lower().startswith(process_name.lower()):
                    log.info("Waiting for %s to finish..", process_name)
                    break
            else:
                break



# Copyright (C) 2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import os
import yaml

from vmcloak.exceptions import SwarmError
from vmcloak.winxp import WindowsXP
from vmcloak.win7 import Windows7x64, Windows7x86
from vmcloak.win81 import Windows81x64, Windows81x86
from vmcloak.win10 import Windows10x64, Windows10x86

class Swarm(object):
    vms = {
        "winxp": {
            "isomount": WindowsXP.mount,
        },
        "win7x86": {
            "isomount": Windows7x86.mount,
            "serialkey": Windows7x86.dummy_serial_key,
        },
        "win7x64": {
            "isomount": Windows7x64.mount,
            "serialkey": Windows7x64.dummy_serial_key,
        },
        "win81x86": {
            "isomount": Windows81x86.mount,
            "serialkey": Windows81x86.dummy_serial_key,
        },
        "win81x64": {
            "isomount": Windows81x64.mount,
            "serialkey": Windows81x64.dummy_serial_key,
        },
        "win10x86": {
            "isomount": Windows10x86.mount,
            "serialkey": Windows10x86.dummy_serial_key,
        },
        "win10x64": {
            "isomount": Windows10x64.mount,
            "serialkey": Windows10x64.dummy_serial_key,
        },
    }

    def __init__(self, confpath):
        self.confpath = confpath
        self.machines = {}
        self.cfg = {}

    def load(self):
        """Reads, parses, interprets, and validates swarm configuration."""
        self.read_swarm()
        self.parse_matrix()
        self.interpret_machines()

    def read_swarm(self):
        """Reads swarm configuration."""
        if not os.path.exists(self.confpath):
            return

        try:
            cfg = yaml.load(open(self.confpath, "rb"))
        except yaml.YAMLError as e:
            raise SwarmError("Error reading swarm configuration: %s" % e)

        if not isinstance(cfg, dict):
            raise SwarmError("Swarm configuration should be a dictionary!")

        self.cfg = cfg

    def apply_dict(self, target, d):
        """Apply a dictionary to one or more elements."""
        if isinstance(target, list):
            for entry in target:
                entry.update(d)
            return target

        if isinstance(target, dict):
            target.update(d)
            return target

        raise SwarmError(
            "Invalid target type for applying a dictionary"
        )

    def parse_matrix(self):
        if not self.cfg.get("matrix"):
            raise SwarmError(
                "Build matrix is missing from swarm configuration!"
            )

        if isinstance(self.cfg["matrix"], dict):
            for name, machine in self.cfg["matrix"].items():
                if name in self.cfg:
                    raise SwarmError(
                        "Can't declare a VM name in the build matrix that "
                        "also exists as configuration entry: '%s'." % name
                    )

                self.machines[name] = self.parse_machine(name, machine)
                self.apply_dict(self.machines[name], machine)
        elif isinstance(self.cfg["matrix"], list):
            self.machines = self.parse_machines(self.cfg["matrix"])
        else:
            raise SwarmError("Build matrix should be a dictionary or list!")

    def parse_machines(self, machines):
        ret = {}
        for machine in machines:
            if machine not in self.cfg:
                raise SwarmError("Machine '%s' is not defined!" % machine)

            ret[machine] = self.parse_machine(machine, self.cfg[machine])
        return ret

    def parse_machine(self, machine, m):
        if "os" not in m:
            raise SwarmError(
                "Machine '%s' does not have an OS defined!" % machine
            )

        ret = {
            "os": m["os"],
        }

        if m["os"] in self.vms:
            ret.update(self.vms[m["os"]])
        else:
            ret.update(self.parse_machine(m["os"]))

        ret["deps"] = self.parse_dependencies(m.get("deps"), ret.get("deps"))
        return ret

    def parse_dependencies(self, deps, inherited):
        ret = []
        ret.extend(inherited or [])

        deps = deps or []
        if isinstance(deps, basestring):
            deps = deps.split()

        for dep in deps:
            if isinstance(dep, basestring):
                ret.append(self.parse_dependency(dep))
            elif isinstance(dep, dict):
                ret.append(self.apply_dict(
                    self.parse_dependency(dep.keys()[0]),
                    dep.values()[0][0]
                ))
            else:
                raise SwarmError()
        return ret

    def parse_dependency(self, dependency):
        if dependency not in self.cfg:
            raise SwarmError("Dependency '%s' not found!" % dependency)

        dep = self.cfg[dependency]
        if isinstance(dep, dict):
            dep = dep.items()
        elif isinstance(dep, list):
            dep = (None, dep),

        ret = []
        for requirement, info in dep:
            if requirement:
                if not requirement.startswith("os:"):
                    raise SwarmError(
                        "Currently only the os: requirement is allowed for "
                        "dependency criteria! Found: '%s'" % requirement
                    )

                target_os = requirement[3:]
            else:
                target_os = None

            if isinstance(info, basestring):
                ret.append({
                    "os": target_os,
                    "dependency": dependency,
                    "version": info,
                })
                continue

            if isinstance(info, list):
                for entry in info:
                    if isinstance(entry, dict):
                        ret.append(self.apply_dict({
                            "os": target_os,
                            "dependency": dependency,
                        }, entry))
                        continue

                    if isinstance(entry, (int, float, basestring)):
                        ret.append({
                            "os": target_os,
                            "dependency": dependency,
                            "version": "%s" % entry,
                        })
                        continue

                    raise SwarmError(
                        "Invalid dependency specification: %r" % info
                    )
                continue

            if isinstance(info, dict):
                ret.append(self.apply_dict({
                    "os": target_os,
                    "dependency": dependency,
                }, info))

        return ret

    def interpret_machines(self):
        """Interprets the machine configuration by removing incompatible
        dependencies."""
        for name, machine in self.machines.items():
            deps = []
            for dep in machine["deps"]:
                versions = []
                for version in dep:
                    if not version["os"] or version["os"] == machine["os"]:
                        versions.append(version)
                if versions:
                    deps.append(versions)
            machine["deps"] = deps



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import logging
import os.path

from vmcloak.abstract import Dependency

log = logging.getLogger(__name__)

class Extract(Dependency):
    name = "extract"

    def init(self):
        self.zip = None
        self.dir = None

    def check(self):
        if not self.zip or not os.path.isfile(self.zip):
            log.error("Please provide zip file to extract.")
            return False

        if not self.dir:
            log.error("Please provide %USERPROFILE% folder to extract to.")
            return False

    def run(self):
        dirpath = os.path.join(self.a.environ("USERPROFILE"), self.dir)
        self.a.extract(dirpath, self.zip)



#!/usr/bin/env python
# Copyright (C) 2010-2015 Cuckoo Foundation.
# This file is part of Cuckoo Sandbox - http://www.cuckoosandbox.org
# See the file 'docs/LICENSE' for copying permission.

import argparse
import cgi
import io
import json
import os
import platform
import re
import shutil
import stat
import subprocess
import sys
import tempfile
import traceback
import zipfile

import SimpleHTTPServer
import SocketServer

AGENT_VERSION = "0.5"
AGENT_FEATURES = [
    "execpy", "pinning", "logs",
]

sys.stdout = io.BytesIO()
sys.stderr = io.BytesIO()

class MiniHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
    server_version = "Cuckoo Agent"

    def do_GET(self):
        request.client_ip, request.client_port = self.client_address
        request.form = {}
        request.files = {}

        if "client_ip" not in state or request.client_ip == state["client_ip"]:
            self.httpd.handle(self)

    def do_POST(self):
        environ = {
            "REQUEST_METHOD": "POST",
            "CONTENT_TYPE": self.headers["Content-Type"],
        }

        form = cgi.FieldStorage(fp=self.rfile,
                                headers=self.headers,
                                environ=environ)

        request.form = {}
        request.files = {}

        # Another pretty fancy workaround. Since we provide backwards
        # compatibility with the Old Agent we will get an xmlrpc request
        # from the analyzer when the analysis has finished. Now xmlrpc being
        # xmlrpc we're getting text/xml as content-type which cgi does not
        # handle. This check detects when there is no available data rather
        # than getting a hard exception trying to do so.
        if form.list:
            for key in form.keys():
                value = form[key]
                if value.filename:
                    request.files[key] = value.file
                else:
                    request.form[key] = value.value

        if "client_ip" not in state or request.client_ip == state["client_ip"]:
            self.httpd.handle(self)

class MiniHTTPServer(object):
    def __init__(self):
        self.handler = MiniHTTPRequestHandler

        # Reference back to the server.
        self.handler.httpd = self

        self.routes = {
            "GET": [],
            "POST": [],
        }

    def run(self, host="0.0.0.0", port=8000):
        self.s = SocketServer.TCPServer((host, port), self.handler)
        self.s.allow_reuse_address = True
        self.s.serve_forever()

    def route(self, path, methods=["GET"]):
        def register(fn):
            for method in methods:
                self.routes[method].append((re.compile(path + "$"), fn))
            return fn
        return register

    def handle(self, obj):
        for route, fn in self.routes[obj.command]:
            if route.match(obj.path):
                ret = fn()
                break
        else:
            ret = json_error(404, message="Route not found")

        ret.init()
        obj.send_response(ret.status_code)
        ret.headers(obj)
        obj.end_headers()

        if isinstance(ret, jsonify):
            obj.wfile.write(ret.json())
        elif isinstance(ret, send_file):
            ret.write(obj.wfile)

    def shutdown(self):
        # BaseServer also features a .shutdown() method, but you can't use
        # that from the same thread as that will deadlock the whole thing.
        self.s._BaseServer__shutdown_request = True

class jsonify(object):
    """Wrapper that represents Flask.jsonify functionality."""
    def __init__(self, **kwargs):
        self.status_code = 200
        self.values = kwargs

    def init(self):
        pass

    def json(self):
        return json.dumps(self.values)

    def headers(self, obj):
        pass

class send_file(object):
    """Wrapper that represents Flask.send_file functionality."""
    def __init__(self, path):
        self.path = path
        self.status_code = 200

    def init(self):
        if not os.path.isfile(self.path):
            self.status_code = 404
            self.length = 0
        else:
            self.length = os.path.getsize(self.path)

    def write(self, sock):
        if not self.length:
            return

        with open(self.path, "rb") as f:
            while True:
                buf = f.read(1024 * 1024)
                if not buf:
                    break

                sock.write(buf)

    def headers(self, obj):
        obj.send_header("Content-Length", self.length)

class request(object):
    form = {}
    files = {}
    client_ip = None
    client_port = None
    environ = {
        "werkzeug.server.shutdown": lambda: app.shutdown(),
    }

app = MiniHTTPServer()
state = {}

def json_error(error_code, message):
    r = jsonify(message=message, error_code=error_code)
    r.status_code = error_code
    return r

def json_exception(message):
    r = jsonify(message=message, error_code=500,
                traceback=traceback.format_exc())
    r.status_code = 500
    return r

def json_success(message, **kwargs):
    return jsonify(message=message, **kwargs)

@app.route("/")
def get_index():
    return json_success(
        "Cuckoo Agent!", version=AGENT_VERSION, features=AGENT_FEATURES
    )

@app.route("/status")
def get_status():
    return json_success("Analysis status",
                        status=state.get("status"),
                        description=state.get("description"))

@app.route("/status", methods=["POST"])
def put_status():
    if "status" not in request.form:
        return json_error("No status has been provided")

    state["status"] = request.form["status"]
    state["description"] = request.form.get("description")
    return json_success("Analysis status updated")

@app.route("/logs")
def get_logs():
    return json_success(
        "Agent logs",
        stdout=sys.stdout.getvalue(),
        stderr=sys.stderr.getvalue()
    )

@app.route("/system")
def get_system():
    return json_success("System", system=platform.system())

@app.route("/environ")
def get_environ():
    return json_success("Environment variables", environ=dict(os.environ))

@app.route("/path")
def get_path():
    return json_success("Agent path", filepath=os.path.abspath(__file__))

@app.route("/mkdir", methods=["POST"])
def do_mkdir():
    if "dirpath" not in request.form:
        return json_error(400, "No dirpath has been provided")

    mode = int(request.form.get("mode", 0777))

    try:
        os.makedirs(request.form["dirpath"], mode=mode)
    except:
        return json_exception("Error creating directory")

    return json_success("Successfully created directory")

@app.route("/mktemp", methods=["GET", "POST"])
def do_mktemp():
    suffix = request.form.get("suffix", "")
    prefix = request.form.get("prefix", "tmp")
    dirpath = request.form.get("dirpath")

    fd, filepath = tempfile.mkstemp(suffix=suffix, prefix=prefix, dir=dirpath)
    os.close(fd)

    return json_success("Successfully created temporary file",
                        filepath=filepath)

@app.route("/mkdtemp", methods=["GET", "POST"])
def do_mkdtemp():
    suffix = request.form.get("suffix", "")
    prefix = request.form.get("prefix", "tmp")
    dirpath = request.form.get("dirpath")

    dirpath = tempfile.mkdtemp(suffix=suffix, prefix=prefix, dir=dirpath)
    return json_success("Successfully created temporary directory",
                        dirpath=dirpath)

@app.route("/store", methods=["POST"])
def do_store():
    if "filepath" not in request.form:
        return json_error(400, "No filepath has been provided")

    if "file" not in request.files:
        return json_error(400, "No file has been provided")

    try:
        with open(request.form["filepath"], "wb") as f:
            f.write(request.files["file"].read())
    except:
        return json_exception("Error storing file")

    return json_success("Successfully stored file")

@app.route("/retrieve", methods=["POST"])
def do_retrieve():
    if "filepath" not in request.form:
        return json_error(400, "No filepath has been provided")

    return send_file(request.form["filepath"])

@app.route("/extract", methods=["POST"])
def do_extract():
    if "dirpath" not in request.form:
        return json_error(400, "No dirpath has been provided")

    if "zipfile" not in request.files:
        return json_error(400, "No zip file has been provided")

    try:
        with zipfile.ZipFile(request.files["zipfile"], "r") as archive:
            archive.extractall(request.form["dirpath"])
    except:
        return json_exception("Error extracting zip file")

    return json_success("Successfully extracted zip file")

@app.route("/remove", methods=["POST"])
def do_remove():
    if "path" not in request.form:
        return json_error(400, "No path has been provided")

    try:
        if os.path.isdir(request.form["path"]):
            # Mark all files as readable so they can be deleted.
            for dirpath, _, filenames in os.walk(request.form["path"]):
                for filename in filenames:
                    os.chmod(os.path.join(dirpath, filename), stat.S_IWRITE)

            shutil.rmtree(request.form["path"])
            message = "Successfully deleted directory"
        elif os.path.isfile(request.form["path"]):
            os.chmod(request.form["path"], stat.S_IWRITE)
            os.remove(request.form["path"])
            message = "Successfully deleted file"
        else:
            return json_error(404, "Path provided does not exist")
    except:
        return json_exception("Error removing file or directory")

    return json_success(message)

@app.route("/execute", methods=["POST"])
def do_execute():
    if "command" not in request.form:
        return json_error(400, "No command has been provided")

    # Execute the command asynchronously? As a shell command?
    async = "async" in request.form
    shell = "shell" in request.form

    cwd = request.form.get("cwd")
    stdout = stderr = None

    try:
        if async:
            subprocess.Popen(request.form["command"], shell=shell, cwd=cwd)
        else:
            p = subprocess.Popen(request.form["command"], shell=shell, cwd=cwd,
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.PIPE)
            stdout, stderr = p.communicate()
    except:
        return json_exception("Error executing command")

    return json_success("Successfully executed command",
                        stdout=stdout, stderr=stderr)

@app.route("/execpy", methods=["POST"])
def do_execpy():
    if "filepath" not in request.form:
        return json_error(400, "No Python file has been provided")

    # Execute the command asynchronously? As a shell command?
    async = "async" in request.form

    cwd = request.form.get("cwd")
    stdout = stderr = None

    args = [
        sys.executable,
        request.form["filepath"],
    ]

    try:
        if async:
            subprocess.Popen(args, cwd=cwd)
        else:
            p = subprocess.Popen(args, cwd=cwd,
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.PIPE)
            stdout, stderr = p.communicate()
    except:
        return json_exception("Error executing command")

    return json_success("Successfully executed command",
                        stdout=stdout, stderr=stderr)

@app.route("/pinning")
def do_pinning():
    if "client_ip" in state:
        return json_error(500, "Agent has already been pinned to an IP!")

    state["client_ip"] = request.client_ip
    return json_success("Successfully pinned Agent",
                        client_ip=request.client_ip)

@app.route("/kill")
def do_kill():
    shutdown = request.environ.get("werkzeug.server.shutdown")
    if shutdown is None:
        return json_error(500, "Not running with the Werkzeug server")

    shutdown()
    return json_success("Quit the Cuckoo Agent")

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("host", nargs="?", default="0.0.0.0")
    parser.add_argument("port", nargs="?", default="8000")
    args = parser.parse_args()

    app.run(host=args.host, port=int(args.port))



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import os.path
import re

from vmcloak.constants import VMCLOAK_ROOT

def valid_serial_key(serial_key):
    """Determines whether `serial_key` has a valid encoding."""
    parts = serial_key.split("-")
    if len(parts) != 5:
        return False

    for part in parts:
        if not re.match("[A-Z0-9]{5}$", part):
            return False

    return True

def valid_keyboard_layout(kblayout):
    kblayout_txt = os.path.join(VMCLOAK_ROOT, "data", "winxp",
                                "keyboard_layout_values.txt")
    for layout in open(kblayout_txt, "rb"):
        if layout.strip() == kblayout:
            return True



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.


VBOX_CONFIG = {
    "VBoxInternal/Devices/pcbios/0/Config": dict(

        # http://blog.prowling.nu/2012/08/modifying-virtualbox-settings-for.html
        DmiBIOSVendor=("bios", "vendor"),
        DmiBIOSVersion=("bios", "version"),
        DmiBIOSReleaseDate=("bios", "release_date"),
        # DmiBIOSReleaseMajor=
        # DmiBIOSReleaseMinor=
        # DmiBIOSFirmwareMajor=
        # DmiBIOSFirmwareMinor=

        DmiSystemVendor=("system", "vendor"),
        DmiSystemProduct=("system", "product"),
        DmiSystemVersion=("system", "version"),
        DmiSystemSerial=("system", "serial"),
        DmiSystemSKU=("system", "sku"),
        DmiSystemFamily=("system", "family"),
        DmiSystemUuid=("system", "uuid"),

        # http://blog.prowling.nu/2012/10/modifying-virtualbox-settings-for.html
        DmiBoardVendor=("board", "vendor"),
        DmiBoardProduct=("board", "product"),
        DmiBoardVersion=("board", "version"),
        DmiBoardSerial=("board", "serial"),
        DmiBoardAssetTag=("board", "asset"),
        DmiBoardLocInChass=("board", "location"),

        DmiChassisVendor=("chassis", "vendor"),
        DmiChassisVersion=("chassis", "version"),
        DmiChassisSerial=("chassis", "serial"),
        DmiChassisAssetTag=("chassis", "asset"),
    ),
    "VBoxInternal/Devices/piix3ide/0/Config": {
        "Port0": dict(

            # http://downloads.cuckoosandbox.org/slides/blackhat.pdf, Page 82
            # https://forums.virtualbox.org/viewtopic.php?f=1&t=48718
            # ATAPIProductId="",
            # ATAPIRevision="",
            # ATAPIVendorId="",
        ),
        "PrimaryMaster": dict(

            # http://blog.prowling.nu/2012/08/modifying-virtualbox-settings-for.html
            SerialNumber=("harddisk", "serial"),
            FirmwareRevision=("harddisk", "revision"),
            ModelNumber=("harddisk", "model"),
        ),
    },
}



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import WindowsAutounattended

class Windows81(WindowsAutounattended):
    name = "win81"
    service_pack = 2
    interface = "Ethernet"

    # List of preferences when multiple Windows 8.1 types are available.
    preference = "pro", "enterprise", "home"

    dummy_serial_key = "GCRJD-8NW9H-F2CDX-CCM8D-9D6T9"

class Windows81x64(Windows81):
    arch = "amd64"
    mount = "/mnt/win81x64", "/mnt/win81"

class Windows81x86(Windows81):
    arch = "x86"
    mount = "/mnt/win81x86", "/mnt/win81"



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import requests
import logging

from StringIO import StringIO

from vmcloak.misc import wait_for_host

log = logging.getLogger(__name__)

class Agent(object):
    def __init__(self, ipaddr, port):
        self.ipaddr = ipaddr
        self.port = port

    def get(self, method, *args, **kwargs):
        """Wrapper around GET requests."""
        url = "http://%s:%s%s" % (self.ipaddr, self.port, method)
        return requests.get(url, *args, **kwargs)

    def post(self, method, **kwargs):
        """Wrapper around POST requests."""
        url = "http://%s:%s%s" % (self.ipaddr, self.port, method)
        return requests.post(url, data=kwargs)

    def postfile(self, method, files, **kwargs):
        """Wrapper around POST requests with attached files."""
        url = "http://%s:%s%s" % (self.ipaddr, self.port, method)
        return requests.post(url, files=files, data=kwargs)

    def ping(self):
        """Ping the machine."""
        return self.get("/", timeout=5)

    def environ(self, value=None, default=None):
        """Obtain one or all environment variable(s)."""
        environ = self.get("/environ").json()["environ"]
        return environ if value is None else environ.get(value, default)

    def execute(self, command, async=False):
        """Execute a command."""
        if async:
            return self.post("/execute", command=command, async="true")
        else:
            return self.post("/execute", command=command)

    def remove(self, path):
        """Remove a file or entire directory."""
        self.post("/remove", path=path)

    def extract(self, dirpath, zipfile):
        """Extract a zip file to folder."""
        zipfile = open(zipfile, "rb")
        self.postfile("/extract", {"zipfile": zipfile}, dirpath=dirpath)

    def shutdown(self):
        """Power off the machine."""
        self.execute("shutdown -s -t 0", async=True)

    def reboot(self):
        """Reboot the machine."""
        self.execute("shutdown -r -t 0", async=True)

    def kill(self):
        """Kill the Agent."""
        self.get("/kill")

    def killprocess(self, process_name):
        """Terminate a process."""
        self.execute("taskkill /F /IM %s" % process_name)

    def hostname(self, hostname):
        """Assign a new hostname."""
        cmdline = "wmic computersystem where name=\"%(oldname)s\" " \
            "call rename name=\"%(newname)s\""
        args = dict(oldname=self.environ("COMPUTERNAME"), newname=hostname)

        # self.execute(cmdline % args, shell=True)
        self.execute(cmdline % args)

    def static_ip(self, ipaddr, netmask, gateway, interface):
        """Change the IP address of this machine."""
        command = (
            "netsh interface ip set address name=\"%s\" static %s %s %s 1"
        ) % (interface, ipaddr, netmask, gateway)

        try:
            requests.post("http://%s:%s/execute" % (self.ipaddr, self.port),
                          data={"command": command}, timeout=5)
        except requests.exceptions.ReadTimeout:
            pass

        # Now wait until the Agent is reachable on the new IP address.
        wait_for_host(ipaddr, self.port)
        self.ipaddr = ipaddr

    def dns_server(self, ipaddr):
        """Set the IP address of the DNS server."""
        command = \
            "netsh interface ip set dns " \
            "name=\"Local Area Connection\" static %s" % ipaddr
        self.execute(command)

    def upload(self, filepath, contents):
        """Upload a file to the Agent."""
        if isinstance(contents, basestring):
            contents = StringIO(contents)
        self.postfile("/store", {"file": contents}, filepath=filepath)

    def click(self, window_title, button_name):
        """Identify a window by its title and click one of its buttons."""
        log.debug("Clicking window '%s' button '%s'", window_title, button_name)
        self.execute("C:\\vmcloak\\click.exe \"%s\" \"%s\"" % (
            window_title, button_name))

    def click_async(self, window_title, button_name):
        """Identify a window by its title and click one of its buttons
        asynchronously. This is mostly used in cases where the click may or
        may not be required, leaving the clicking process hanging."""
        log.debug("Clicking (async) window '%s' button '%s'", window_title, button_name)
        self.execute("C:\\vmcloak\\click.exe \"%s\" \"%s\"" % (
            window_title, button_name), async=True)

    def resolution(self, width, height):
        """Set the screen resolution of this Virtual Machine."""
        self.execute("C:\\Python27\\python.exe C:\\vmcloak\\resolution.py "
                     "%s %s" % (width, height))



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import Dependency

class DotNet(Dependency):
    name = "dotnet"
    depends = "wic"
    default = "4.0"
    recommended = True
    exes = [
        {
            "version": "4.0",
            "url": "https://cuckoo.sh/vmcloak/dotNetFx40_Full_x86_x64.exe",
            "sha1": "58da3d74db353aad03588cbb5cea8234166d8b99",
        },
        {
            "version": "4.5.1",
            "url": "https://cuckoo.sh/vmcloak/NDP451-KB2858728-x86-x64-AllOS-ENU.exe",
            "sha1": "5934dd101414bbc0b7f1ee2780d2fc8b9bec5c4d",
        },
        {
            "version": "4.6.1",
            "url": "https://cuckoo.sh/vmcloak/NDP461-KB3102436-x86-x64-AllOS-ENU.exe",
            "sha1": "83d048d171ff44a3cad9b422137656f585295866",
        },
    ]

    def run(self):
        self.upload_dependency("C:\\setup.exe")
        self.a.execute("C:\\setup.exe /passive /norestart")
        self.a.remove("C:\\setup.exe")

class DotNet40(DotNet, Dependency):
    """Backwards compatibility."""
    name = "dotnet40"
    recommended = False



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import Dependency

class Silverlight(Dependency):
    name = "silverlight"
    exes = [
        {
            "url": "https://cuckoo.sh/vmcloak-files/Silverlight_Developer_x86.exe",
            "sha1": "f87518a85e90050cbe4b4b76308f105c7b37acdc",
        },
    ]

    def run(self):
        self.upload_dependency("C:\\setup.exe")
        self.a.execute("C:\\setup.exe /q /doNotRequireDRMPrompt /noupdate")
        self.a.remove("C:\\setup.exe")



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import os

from sqlalchemy import create_engine, Column, ForeignKey
from sqlalchemy import Integer, Text, String
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

conf_path = os.path.join(os.getenv("HOME"), ".vmcloak")
image_path = os.path.join(conf_path, "image")
vms_path = os.path.join(conf_path, "vms")
deps_path = os.path.join(conf_path, "deps")
iso_dst_path = os.path.join(conf_path, "iso")

repository = os.path.join(conf_path, "repository.db")
engine = create_engine("sqlite:///%s" % repository)
Base = declarative_base()
Session = sessionmaker(bind=engine)

class Image(Base):
    """Represents each image that is created and altered along the way."""
    __tablename__ = "image"

    id = Column(Integer, primary_key=True)
    mode = Column(String(16))
    vm = Column(String(16))
    name = Column(String(64))
    path = Column(Text)
    osversion = Column(String(32))
    servicepack = Column(String(32))
    ipaddr = Column(String(32))
    port = Column(Integer)
    adapter = Column(String(32))
    netmask = Column(String(32))
    gateway = Column(String(32))
    cpus = Column(Integer)
    ramsize = Column(Integer)
    vramsize = Column(Integer)

class Snapshot(Base):
    """Represents each snapshot that has been created."""
    __tablename__ = "snapshot"

    id = Column(Integer, primary_key=True)
    image_id = Column(Integer, ForeignKey("image.id"))
    vmname = Column(String(64))
    ipaddr = Column(String(32))
    port = Column(Integer)
    hostname = Column(String(32))

if not os.path.isdir(conf_path):
    os.mkdir(conf_path)

Base.metadata.create_all(engine)

if not os.path.isdir(image_path):
    os.mkdir(image_path)

if not os.path.isdir(deps_path):
    os.mkdir(deps_path)

if not os.path.isdir(iso_dst_path):
    os.mkdir(iso_dst_path)



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import click
import logging
import os.path
import shutil
import tempfile
import time

from sqlalchemy.orm.session import make_transient

import vmcloak.dependencies

from vmcloak.agent import Agent
from vmcloak.dependencies import Python27
from vmcloak.exceptions import DependencyError
from vmcloak.misc import wait_for_host, register_cuckoo, drop_privileges
from vmcloak.misc import ipaddr_increase
from vmcloak.rand import random_string
from vmcloak.repository import image_path, Session, Image, Snapshot, iso_dst_path
from vmcloak.winxp import WindowsXP
from vmcloak.win7 import Windows7x86, Windows7x64
from vmcloak.win81 import Windows81x86, Windows81x64
from vmcloak.win10 import Windows10x86, Windows10x64
from vmcloak.vm import VirtualBox
from vmcloak.constants import VMCLOAK_VM_MODES

logging.basicConfig()
log = logging.getLogger("vmcloak")
log.setLevel(logging.ERROR)

def initvm(image, name=None, multi=False, ramsize=None, vramsize=None, cpus=None):
    handlers = {
        "winxp": WindowsXP,
        "win7x86": Windows7x86,
        "win7x64": Windows7x64,
        "win81x86": Windows81x86,
        "win81x64": Windows81x64,
        "win10x86": Windows10x86,
        "win10x64": Windows10x64,
    }

    h = handlers[image.osversion]
    m = None

    if image.vm == "virtualbox":
        m = VirtualBox(name=name or image.name)
        m.create_vm()
        m.os_type(image.osversion)
        m.cpus(cpus or image.cpus)
        m.mouse("usbtablet")
        m.ramsize(ramsize or image.ramsize)
        m.vramsize(vramsize or image.vramsize)
        m.attach_hd(image.path, multi=multi)
        # Ensure the slot is at least allocated for by an empty drive.
        m.detach_iso()
        m.hostonly(nictype=h.nictype, adapter=image.adapter)

    return m, h

@click.group(invoke_without_command=True)
@click.option("-u", "--user", help="Drop privileges to user.")
def main(user):
    user and drop_privileges(user)

@main.command()
@click.argument("name")
@click.argument("outname")
def clone(name, outname):
    session = Session()

    image = session.query(Image).filter_by(name=name).first()
    if not image:
        log.error("Image not found: %s", name)
        exit(1)

    outpath = os.path.join(image_path, "%s.vdi" % outname)

    m = VirtualBox(None)
    m.clone_hd(image.path, outpath)

    # Retain all fields but update the mode, name & path.
    make_transient(image)
    image.id = None
    image.mode = "normal"
    image.name = outname
    image.path = outpath

    session.add(image)
    session.commit()

@main.command()
@click.argument("name")
@click.option("--winxp", is_flag=True, help="This is a Windows XP instance.")
@click.option("--win7x86", is_flag=True, help="This is a Windows 7 32-bit instance.")
@click.option("--win7x64", is_flag=True, help="This is a Windows 7 64-bit instance.")
@click.option("--win81x86", is_flag=True, help="This is a Windows 8.1 32-bit instance.")
@click.option("--win81x64", is_flag=True, help="This is a Windows 8.1 64-bit instance.")
@click.option("--win10x86", is_flag=True, help="This is a Windows 10 32-bit instance.")
@click.option("--win10x64", is_flag=True, help="This is a Windows 10 64-bit instance.")
@click.option("--product", help="Windows 7 product version.")
@click.option("--vm", default="virtualbox", help="Virtual Machinery.")
@click.option("--iso-mount", help="Mounted ISO Windows installer image.")
@click.option("--serial-key", help="Windows Serial Key.")
@click.option("--ip", default="192.168.56.2", help="Guest IP address.")
@click.option("--port", default=8000, help="Port to run the Agent on.")
@click.option("--adapter", default="vboxnet0", help="Network adapter.")
@click.option("--netmask", default="255.255.255.0", help="Guest IP address.")
@click.option("--gateway", default="192.168.56.1", help="Guest IP address.")
@click.option("--dns", default="8.8.8.8", help="DNS Server.")
@click.option("--cpus", default=1, help="CPU count.")
@click.option("--ramsize", type=int, help="Memory size")
@click.option("--vramsize", default=16, help="Video memory size")
@click.option("--tempdir", default=iso_dst_path, help="Temporary directory to build the ISO file.")
@click.option("--resolution", default="1024x768", help="Screen resolution.")
@click.option("--vm-visible", is_flag=True, help="Start the Virtual Machine in GUI mode.")
@click.option("-d", "--debug", is_flag=True, help="Install Virtual Machine in debug mode.")
@click.option("-v", "--verbose", is_flag=True, help="Verbose logging.")
def init(name, winxp, win7x86, win7x64, win81x86, win81x64, win10x86,
         win10x64, product, vm, iso_mount, serial_key, ip, port, adapter,
         netmask, gateway, dns, cpus, ramsize, vramsize, tempdir, resolution,
         vm_visible, debug, verbose):
    if verbose:
        log.setLevel(logging.INFO)
    if debug:
        log.setLevel(logging.DEBUG)

    session = Session()
    image = session.query(Image).filter_by(name=name).first()
    if image:
        log.error("Image already exists: %s", name)
        exit(1)

    if vm not in VMCLOAK_VM_MODES:
        log.error("Only VirtualBox Machinery or iso is supported at this point.")
        exit(1)

    if winxp:
        h = WindowsXP()
        osversion = "winxp"
        ramsize = ramsize or 1024
    elif win7x86:
        h = Windows7x86()
        ramsize = ramsize or 1024
        osversion = "win7x86"
    elif win7x64:
        h = Windows7x64()
        ramsize = ramsize or 2048
        osversion = "win7x64"
    elif win81x86:
        h = Windows81x86()
        ramsize = ramsize or 2048
        osversion = "win81x86"
    elif win81x64:
        h = Windows81x64()
        ramsize = ramsize or 2048
        osversion = "win81x64"
    elif win10x86:
        h = Windows10x86()
        ramsize = ramsize or 2048
        osversion = "win10x86"
    elif win10x64:
        h = Windows10x64()
        ramsize = ramsize or 2048
        osversion = "win10x64"
    else:
        log.error(
            "Please provide one of --winxp, --win7x86, --win7x64, "
            "--win81x86, --win81x64, --win10x86, --win10x64."
        )
        exit(1)

    mount = h.pickmount(iso_mount)
    if not mount:
        log.error("Please specify --iso-mount to a directory containing the "
                  "mounted Windows Installer ISO image.")
        log.info("Refer to the documentation on mounting an .iso image.")
        exit(1)

    if not h.set_serial_key(serial_key):
        exit(1)

    h.configure(tempdir=tempdir, product=product)

    reso_width, reso_height = resolution.split("x")

    settings = dict(
        GUEST_IP=ip,
        AGENT_PORT=port,
        GUEST_MASK=netmask,
        GUEST_GATEWAY=gateway,
        DNSSERVER=dns,
        DEBUG="yes" if debug else "no",
        RESO_WIDTH=reso_width,
        RESO_HEIGHT=reso_height,
        INTERFACE=h.interface,
    )

    bootstrap = tempfile.mkdtemp(dir=tempdir)

    vmcloak_dir = os.path.join(bootstrap, "vmcloak")
    os.mkdir(vmcloak_dir)

    # Write the configuration values for bootstrap.bat.
    with open(os.path.join(vmcloak_dir, "settings.bat"), "wb") as f:
        for key, value in settings.items():
            print>>f, "set %s=%s" % (key, value)

    # Download the Python dependency and set it up for bootstrapping the VM.
    d = Python27(i=Image(osversion=osversion))
    d.download()
    shutil.copy(d.filepath, vmcloak_dir)

    iso_path = os.path.join(tempdir, "%s.iso" % name)
    hdd_path = os.path.join(image_path, "%s.vdi" % name)
    m = VirtualBox(name=name)

    if not h.buildiso(mount, iso_path, bootstrap, tempdir):
        shutil.rmtree(bootstrap)
        exit(1)

    shutil.rmtree(bootstrap)

    if vm == "virtualbox":
        m.create_vm()
        m.os_type(osversion)
        m.cpus(cpus)
        m.mouse("usbtablet")
        m.ramsize(ramsize)
        m.vramsize(vramsize)
        m.create_hd(hdd_path)
        m.attach_iso(iso_path)
        m.hostonly(nictype=h.nictype, adapter=adapter)

        log.info("Starting the Virtual Machine %r to install Windows.", name)
        m.start_vm(visible=vm_visible)

        m.wait_for_state(shutdown=True)

        m.detach_iso()
        os.unlink(iso_path)

        m.remove_hd()
        m.compact_hd(hdd_path)
        m.delete_vm()
    else:
        log.info("You can find your deployment ISO image from : %s" % iso_path)

    log.info("Added image %r to the repository.", name)
    session.add(Image(name=name, path=hdd_path, osversion=osversion,
                      servicepack="%s" % h.service_pack, mode="normal",
                      ipaddr=ip, port=port, adapter=adapter,
                      netmask=netmask, gateway=gateway,
                      cpus=cpus, ramsize=ramsize, vramsize=vramsize, vm="%s" % vm))
    session.commit()

@main.command()
@click.argument("name")
@click.argument("dependencies", nargs=-1)
@click.option("--vm-visible", is_flag=True)
@click.option("-r", "--recommended", is_flag=True, help="Install recommended packages.")
@click.option("-d", "--debug", is_flag=True, help="Install applications in debug mode.")
def install(name, dependencies, vm_visible, recommended, debug):
    if debug:
        log.setLevel(logging.DEBUG)

    session = Session()

    image = session.query(Image).filter_by(name=name).first()
    if not image:
        log.error("Image not found: %s", name)
        exit(1)

    if image.mode != "normal":
        log.error("You can't install dependencies in this image as you have "
                  "already made snapshots with it!")
        log.error("Please vmcloak-clone it and update the clone.")
        exit(1)

    m, h = initvm(image)

    if image.vm == "virtualbox":
        m.start_vm(visible=vm_visible)

    wait_for_host(image.ipaddr, image.port)

    a = Agent(image.ipaddr, image.port)
    a.ping()

    settings = {}
    deps = []

    # Include all recommended dependencies if requested.
    for dependency in vmcloak.dependencies.plugins:
        if recommended and dependency.recommended:
            deps.append((dependency.name, dependency.default))

    # Fetch the configuration settings off of the arguments.
    for dependency in dependencies:
        if "." in dependency and "=" in dependency:
            key, value = dependency.split("=", 1)
            settings[key.strip()] = value.strip()
        elif ":" in dependency:
            dependency, version = dependency.split(":", 1)
            deps.append((dependency, version))
        else:
            deps.append((dependency, None))

    for dependency, version in deps:
        if dependency not in vmcloak.dependencies.names:
            log.error("Unknown dependency %s..", dependency)
            break

        if version:
            log.info("Installing dependency %s %s..", dependency, version)
        else:
            log.info("Installing dependency %s..", dependency)

        try:
            # TODO Recursive install function.
            d = vmcloak.dependencies.names[dependency]

            # Check if there are any "childs" for dependencies.
            if d.depends:
                depends = d.depends
                if isinstance(depends, basestring):
                    depends = [depends]

                for depend in depends:
                    if ":" in depend:
                        depend, dversion = depend.split(":", 1)
                    else:
                        dversion = None

                    if dversion:
                        log.info("Installing child dependency %s %s..", depend, dversion)
                    else:
                        log.info("Installing child dependency %s..", depend)

                    # Install dependency child before dependency itself.
                    dd = vmcloak.dependencies.names[depend]
                    dd(h, m, a, image, dversion, settings).run()

                # Reboot the VM as we expect most dependencies to be related
                # to KB installs.
                a.reboot()
                time.sleep(10)
                wait_for_host(image.ipaddr, image.port)

            d(h, m, a, image, version, settings).run()
        except DependencyError:
            log.error("The dependency %s returned an error..", dependency)
            break

    if image.vm == "virtualbox":
        a.shutdown()
        m.wait_for_state(shutdown=True)

        m.remove_hd()
        m.compact_hd(image.path)
        m.delete_vm()
    else:
        a.reboot()

@main.command()
@click.argument("name")
@click.option("--vm-visible", is_flag=True)
def modify(name, vm_visible):
    session = Session()

    image = session.query(Image).filter_by(name=name).first()
    if not image:
        log.error("Image not found: %s", name)
        exit(1)

    if image.mode != "normal":
        log.error("You can't modify this image as you have already made "
                  "snapshots with it!")
        log.error("Please vmcloak-clone it and modify the clone.")
        exit(1)

    m, h = initvm(image)

    m.start_vm(visible=vm_visible)
    wait_for_host(image.ipaddr, image.port)

    log.info("The Virtual Machine has booted and is ready to be modified!")
    log.info("When you shut it down, all changes will be saved.")

    m.wait_for_state(shutdown=True)

    m.remove_hd()
    m.compact_hd(image.path)
    m.delete_vm()

@main.command()
@click.argument("vmname")
@click.argument("cuckoo")
@click.argument("tags", required=False, default="")
def register(vmname, cuckoo, tags):
    session = Session()

    snapshot = session.query(Snapshot).filter_by(vmname=vmname).first()
    if not snapshot:
        log.error("Snapshot not found: %s", vmname)
        exit(1)

    # TODO Add snapshot.port & snapshot.adapter to the configuration.
    # But those options will require various changes in Cuckoo as well.
    register_cuckoo(snapshot.ipaddr, tags, vmname, cuckoo)

def do_snapshot(image, vmname, ipaddr, resolution, ramsize, cpus,
                hostname, adapter, vm_visible):
    m, h = initvm(image, name=vmname, multi=True, ramsize=ramsize, cpus=cpus)

    m.start_vm(visible=vm_visible)

    wait_for_host(image.ipaddr, image.port)
    a = Agent(image.ipaddr, image.port)
    a.ping()

    # Assign a new hostname.
    a.hostname(hostname)
    a.reboot()
    a.kill()

    # Wait for the reboot to kick in.
    time.sleep(10)
    wait_for_host(image.ipaddr, image.port)
    a.ping()

    if resolution:
        width, height = resolution.split("x")
        a.resolution(width, height)

    a.remove("C:\\vmcloak")
    a.static_ip(ipaddr, image.netmask, image.gateway, h.interface)

    m.snapshot("vmcloak", "Snapshot created by VM Cloak.")
    m.stopvm()

    # Create a database entry for this snapshot.
    snapshot = Snapshot(image_id=image.id, vmname=vmname, ipaddr=ipaddr,
                        port=image.port, hostname=hostname)
    return snapshot

@main.command()
@click.argument("name")
@click.argument("vmname")
@click.argument("ipaddr", required=False, default="192.168.56.101")
@click.option("--resolution", help="Screen resolution.")
@click.option("--ramsize", type=int, help="Amount of virtual memory to assign.")
@click.option("--cpus", type=int, help="Amount of CPUs to assign.")
@click.option("--hostname", help="Hostname for this VM.")
@click.option("--adapter", help="Hostonly adapter for this VM.")
@click.option("--vm-visible", is_flag=True, help="Start the Virtual Machine in GUI mode.")
@click.option("--count", type=int, help="The amount of snapshots to make.")
@click.option("-d", "--debug", is_flag=True, help="Make snapshot in debug mode.")
def snapshot(name, vmname, ipaddr, resolution, ramsize, cpus, hostname,
             adapter, vm_visible, count, debug):
    if debug:
        log.setLevel(logging.DEBUG)

    session = Session()

    if adapter:
        log.error(
            "Specifying a different adapter is not yet supported for "
            "snapshots (this will require detaching the current adapter and "
            "attaching a new one after the static IP address has been "
            "updated or so)."
        )
        exit(1)

    image = session.query(Image).filter_by(name=name).first()
    if not image:
        log.error("Image not found: %s", name)
        exit(1)

    # From now on this image is multiattach.
    image.mode = "multiattach"
    session.commit()

    if not count:
        snapshot = do_snapshot(
            image, vmname, ipaddr, resolution, ramsize, cpus,
            hostname or random_string(8, 16), adapter, vm_visible
        )
        session.add(snapshot)
    else:
        if hostname:
            log.error(
                "You specified a hostname, but this is not supported when "
                "creating multiple snapshots at once."
            )
            exit(1)

        for x in xrange(count):
            snapshot = do_snapshot(
                image, "%s%d" % (vmname, x + 1), ipaddr, resolution,
                ramsize, cpus, hostname, adapter, vm_visible
            )
            session.add(snapshot)

            # TODO Implement some limits to make sure that the IP address does
            # not "exceed" its provided subnet (and thus also require the user
            # to specify an IP range, rather than an IP address).
            ipaddr = ipaddr_increase(ipaddr)
            hostname = random_string(8, 16)

    session.commit()

@main.command()
@click.argument("name")
@click.argument("filepath", type=click.Path(writable=True))
def export(name, filepath):
    if not filepath.endswith((".ova", ".ovf")):
        log.error("The exported file should be either .ova or .ovf")
        exit(1)

    session = Session()

    image = session.query(Image).filter_by(name=name).first()
    if not image:
        log.error("Image not found: %s", name)
        exit(1)

    if image.mode != "normal":
        log.error("You can't export this image as you have already made "
                  "snapshots with it!")
        log.error("Please vmcloak clone it and export the clone.")
        exit(1)

    m, h = initvm(image)

    m.export(filepath)

    m.remove_hd()
    m.compact_hd(image.path)
    m.delete_vm()

def list_dependencies():
    print "Name", "version", "target", "sha1"
    print
    for name, d in sorted(vmcloak.dependencies.names.items()):
        if d.exes:
            versionlen = max(len(exe.get("version", "None")) for exe in d.exes)
        else:
            versionlen = 4

        print name
        for exe in d.exes:
            v = exe.get("version", "None")
            print "  *" if d.default and d.default == v else "   ",
            print exe.get("version", "None") + " "*(versionlen - len(v)),
            print exe.get("target"), exe["sha1"]
        print

@main.group("list")
def _list():
    pass

@_list.command("dependencies")
def _list_dependencies():
    list_dependencies()

@_list.command("deps")
def _list_deps():
    list_dependencies()



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import time

from vmcloak.abstract import Dependency

class KB(Dependency):
    name = "kb"
    exes = [
        {
            "version": "2729094",
            "target": "win7x64",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2729094-v2-x64.msu",
            "sha1": "e1a3ecc5030a51711f558f90dd1db52e24ce074b",
        },
        {
            "version": "2729094",
            "target": "win7x86",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2729094-v2-x86.msu",
            "sha1": "565e7f2a6562ace748c5b6165aa342a11c96aa98",
        },
        {
            "version": "2731771",
            "target": "win7x64",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2731771-x64.msu",
            "sha1": "98dba6673cedbc2860c76b9686e895664d463347",
        },
        {
            "version": "2731771",
            "target": "win7x86",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2731771-x86.msu",
            "sha1": "86675d2fd327b328793dc179727ce0ce5107a72e",
        },
        {
            "version": "2533623",
            "target": "win7x64",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2533623-x64.msu",
            "sha1": "8a59ea3c7378895791e6cdca38cc2ad9e83bebff",
        },
        {
            "version": "2533623",
            "target": "win7x86",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2533623-x86.msu",
            "sha1": "25becc0815f3e47b0ba2ae84480e75438c119859",
        },
        {
            "version": "2670838",
            "target": "win7x64",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2670838-x64.msu",
            "sha1": "9f667ff60e80b64cbed2774681302baeaf0fc6a6",
        },
        {
            "version": "2670838",
            "target": "win7x86",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2670838-x86.msu",
            "sha1": "984b8d122a688d917f81c04155225b3ef31f012e",
        },
        {
            "version": "2786081",
            "target": "win7x64",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2786081-x64.msu",
            "sha1": "dc63b04c58d952d533c40b185a8b555b50d47905",
        },
        {
            "version": "2786081",
            "target": "win7x86",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2786081-x86.msu",
            "sha1": "70122aca48659bfb8a06bed08cb7047c0c45c5f4",
        },
        {
            "version": "2639308",
            "target": "win7x64",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2639308-x64.msu",
            "sha1": "67eedaf061e02d503028d970515d88d8fe95579d",
        },
        {
            "version": "2639308",
            "target": "win7x86",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2639308-x86.msu",
            "sha1": "96e09ef9caf3907a32315839086b9f576bb46459",
        },
        {
            "version": "2834140",
            "target": "win7x64",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2834140-v2-x64.msu",
            "sha1": "3db9d9b3dc20515bf4b164821b721402e34ad9d6",
        },
        {
            "version": "2834140",
            "target": "win7x86",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2834140-v2-x86.msu",
            "sha1": "b57c05e9da2c66e1bb27868c92db177a1d62b2fb",
        },
        {
            "version": "2882822",
            "target": "win7x64",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2882822-x64.msu",
            "sha1": "ec92821f6ee62ac9d2f74e847f87be0bf9cfcb31",
        },
        {
            "version": "2882822",
            "target": "win7x86",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2882822-x86.msu",
            "sha1": "7fab5b9ca9b5c2395b505d1234ee889941bfb151",
        },
        {
            "version": "2888049",
            "target": "win7x64",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2888049-x64.msu",
            "sha1": "fae6327b151ae04b56fac431e682145c14474c69",
        },
        {
            "version": "2888049",
            "target": "win7x86",
            "url": "https://cuckoo.sh/vmcloak/Windows6.1-KB2888049-x86.msu",
            "sha1": "65b4c7a5773fab177d20c8e49d773492e55e8d76",
        },
    ]

    def run(self):
        self.upload_dependency("C:\\setup.msu")
        self.a.execute("sc config wuauserv start= auto")
        time.sleep(1)
        self.a.execute("wusa.exe C:\\setup.msu /quiet /norestart")

        self.a.remove("C:\\setup.msu")

        self.a.execute("sc config wuauserv start= disabled")
        self.a.execute("net stop wuauserv")



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import Dependency

class IE10(Dependency):
    name = "ie10"
    default = "10"
    depends = [
        "kb:2729094", "kb:2731771", "kb:2533623", "kb:2670838", "kb:2786081",
        "kb:2639308", "kb:2834140",
    ]
    exes = [
        {
            "version": "10",
            "target": "win7x64",
            "url": "https://cuckoo.sh/vmcloak/IE10-Windows6.1-x64-en-us.exe",
            "sha1": "17d1eaca123e453269b12b20863fd5ce96727888",
        },
        {
            "version": "10",
            "target": "win7x86",
            "url": "https://cuckoo.sh/vmcloak/IE10-Windows6.1-x86-en-us.exe",
            "sha1": "e6552da75de95f6b1f7937c2bdb5fe14443dea2a",
        },
    ]

    def run(self):

        self.upload_dependency("C:\\setup.exe")
        self.a.execute("C:\\setup.exe /quiet /norestart /update-no")

        self.a.remove("C:\\setup.exe")



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import io
import logging
import os.path
import random
import requests

from vmcloak.abstract import Dependency

log = logging.getLogger(__name__)

class Wallpaper(Dependency):
    name = "wallpaper"

    doges = [
        "https://cuckoo.sh/vmcloak/doge1.jpg",
        "https://cuckoo.sh/vmcloak/doge2.jpg",
        "https://cuckoo.sh/vmcloak/doge3.jpg",
    ]

    def init(self):
        self.filepath = None

    def run(self):
        uploadpath = os.path.join(
            self.a.environ("USERPROFILE"), "Pictures", "wall.jpg"
        )

        if not self.filepath:
            f = io.BytesIO(requests.get(random.choice(self.doges)).content)
        else:
            f = open(self.filepath, "rb")

        self.a.upload(uploadpath, f)

        # Add Wallpaper in registry.
        self.a.execute(
            "reg add \"HKEY_CURRENT_USER\\Control Panel\\Desktop\" "
            "/v Wallpaper /t REG_SZ /d  %s /f" % uploadpath
        )



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import Dependency

class Pillow(Dependency):
    name = "pillow"
    recommended = True
    exes = [
        {
            "url": "https://cuckoo.sh/vmcloak/Pillow-2.9.0.win32-py2.7.exe",
            "sha1": "1138f6db53b54943cbe7cf237c4df7e9255ca034",
        },
    ]

    def run(self):
        self.upload_dependency("C:\\pillow.exe")
        self.a.execute("C:\\pillow.exe", async=True)
        self.a.click("Setup", "&Next >")
        self.a.click("Setup", "&Next >")
        self.a.click("Setup", "&Next >")
        self.a.click("Setup", "Finish")

        self.wait_process_exit("pillow.exe")
        self.a.remove("C:\\pillow.exe")



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import Dependency

class IE9(Dependency):
    name = "ie9"
    default = "9"
    exes = [
        {
            "version": "9",
            "target": "win7x64",
            "url": "https://cuckoo.sh/vmcloak/IE9-Windows7-x64-enu.exe",
            "sha1": "5ace268e2812793e2232648f62cdf4be17b2b4dd",
        },
        {
            "version": "9",
            "target": "win7x86",
            "url": "https://cuckoo.sh/vmcloak/IE9-Windows7-x86-enu.exe",
            "sha1": "fb2b17cf1d22f3e2b2ad339c5bd78f8fab406d03",
        },
    ]

    def run(self):
        self.upload_dependency("C:\\setup.exe")
        self.a.execute("C:\\setup.exe /quiet /norestart /update-no")
        self.a.remove("C:\\setup.exe")



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import Dependency

class DnsServer(Dependency):
    name = "dns"

    def run(self):
        self.a.dns_server(self.version)



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import logging

from vmcloak.abstract import Dependency
from vmcloak.exceptions import DependencyError

log = logging.getLogger(__name__)

class Resolution(Dependency):
    name = "resolution"

    def init(self):
        if not self.version or "x" not in self.version:
            log.error("Please specify a resolution.")
            raise DependencyError

        width, height = self.version.split("x")
        self.width, self.height = width, height

    def run(self):
        self.a.resolution(self.width, self.height)



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import Dependency

class Winrar(Dependency):
    name = "winrar"
    default = "5.31"
    exes = [
        {
            "version": "5.31_x64",
            "url": "https://cuckoo.sh/vmcloak/winrar-x64-531.exe",
            "sha1": "48add5a966ed940c7d88456caf7a5f5c2a6c27a7",
        },
        {
            "version": "5.31",
            "url": "https://cuckoo.sh/vmcloak/wrar531.exe",
            "sha1": "e19805a1738975aeec19a25a4e461d52eaf9b231",
        },
    ]

    def run(self):
        self.upload_dependency("C:\\%s" % self.filename)
        self.a.execute("C:\\%s /S" % self.filename)
        self.a.remove("C:\\%s" % self.filename)



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.
# Firefox Dependency submitted by Jason Lewis.
# https://download-installer.cdn.mozilla.net/pub/firefox/releases/41.0.2/win32/en-US/Firefox%20Setup%2041.0.2.exe
# file was renamed Firefox_Setup_41.0.2.exe to avoid issues with spaces.

from vmcloak.abstract import Dependency

class Firefox(Dependency):
    name = "firefox"
    exes = [
        {
            "version": "41.0.2",
            "url": "https://cuckoo.sh/vmcloak/Firefox_Setup_41.0.2.exe",
            "sha1": "c5118ca76f0cf6ecda5d2b9292bf191525c9627a",
        },
    ]

    def run(self):
        self.upload_dependency("C:\\Firefox_Setup_41.0.2.exe")
        self.a.execute("C:\\Firefox_Setup_41.0.2.exe -ms")
        self.a.remove("C:\\Firefox_Setup_41.0.2.exe")

class Firefox41(Firefox, Dependency):
    """Backwards compatibility"""
    name = "firefox_41"



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import logging
import os.path

from vmcloak.abstract import Dependency

log = logging.getLogger(__name__)

config = """
<Configuration Product="ProPlus">
    <Display Level="none" CompletionNotice="no" SuppressModal="yes" AcceptEula="yes" />
    <PIDKEY Value="%(serial_key)s" />
    <Setting Id="AUTO_ACTIVATE" Value="%(activate)s" />
</Configuration>
"""

class Office(Dependency):
    name = "office"

    def init(self):
        self.isopath = None
        self.serialkey = None
        self.activate = None

    def check(self):
        if not self.serialkey:
            log.error("Please provide a serial key for Office.")
            return False

        if not self.isopath or not os.path.isfile(self.isopath):
            log.error("Please provide the Office installer ISO file.")
            return False

        if not self.activate:
            self.activate = 0
            log.info("Defaulting activate to False")
            return True
        elif self.activate not in ["0", "1"]:
            log.error(
                "Please keep activate value 0 or 1. You had %s.",
                self.activate
            )
            return False

    def run(self):
        if self.i.vm == "virtualbox":
            self.disable_autorun()
            self.m.attach_iso(self.isopath)

        self.a.upload(
            "C:\\config.xml",
            config % dict(serial_key=self.serialkey, activate=self.activate)
        )
        self.a.execute("D:\\setup.exe /config C:\\config.xml")

        # Wait until setup.exe is no longer running.
        self.wait_process_exit("setup.exe")

        self.a.remove("C:\\config.xml")

        if self.i.vm == "virtualbox":
            self.m.detach_iso()

class Office2007(Office, Dependency):
    """Backwards compatibility."""
    name = "office2007"



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import Dependency

class AdobePdf(Dependency):
    name = "adobepdf"
    default = "9.0.0"
    recommended = True
    exes = [
        {
            "version": "9.0.0",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr90_en_US.exe",
            "sha1": "8faabd08289b9a88023f71136f13fc4bd3290ef0",
        },
        {
            "version": "9.1.0",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr910_en_US.exe",
            "sha1": "1e0db06c84d89c8f58b543a41ec35b133de7ea19",
        },
        {
            "version": "9.2.0",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr920_en_US.exe",
            "sha1": "4b6207b018cf2be2f49d1f045ff369eb3bee88da",
        },
        {
            "version": "9.3.0",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr930_en_US.exe",
            "sha1": "98cacd6069e78a0dd1ef87ce24e59716fecf8aa0",
        },
        {
            "version": "9.3.3",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr933_en_US.exe",
            "sha1": "b1ed1d350db97ddd562606449804e705d4ffe1c7",
        },
        {
            "version": "9.3.4",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr934_en_US.exe",
            "sha1": "e3bb8eff9d199ab1f4b5f7a10e514a74e0384ca0",
        },
        {
            "version": "9.4.0",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr940_en_US.exe",
            "sha1": "4652a454056b2323097a6357292db3af239bb610",
        },
        {
            "version": "9.5.0",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr950_en_US.exe",
            "sha1": "e46000691a6dbcd7892078b46c8ee13613683545",
        },
        {
            "version": "10.1.4",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr1014_en_US.exe",
            "sha1": "fe6808d5d11e94dc5581f33ed386ce552f0c84d6",
        },
        {
            "version": "11.0.2",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr11002_en_US.exe",
            "sha1": "e1d9e57f08e169fb1c925f8ded93e5f5efe5cda3",
        },
        {
            "version": "11.0.3",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr11003_en_US.exe",
            "sha1": "9c2b6903b000ecf2869e1555bc6e1b287e6176bf",
        },
        {
            "version": "11.0.4",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr11004_en_US.exe",
            "sha1": "9c295c16d374735bf292ef6c630c9ab392c22500",
        },
        {
            "version": "11.0.6",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr11006_en_US.exe",
            "sha1": "6a3d5b494b4ed6e11fc7d917afc03eaf05d4a6aa",
        },
        {
            "version": "11.0.7",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr11007_en_US.exe",
            "sha1": "3e08c3f6daad59f463227590cc438b3906648f5e",
        },
        {
            "version": "11.0.8",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr11008_en_US.exe",
            "sha1": "3e889258ea2000337bbe180d81317d44f617a292",
        },
        {
            "version": "11.0.9",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr11009_en_US.exe",
            "sha1": "53b367bff07a63ee07cf1cd090360b75d3fc6bfb",
        },
        {
            "version": "11.0.10",
            "url": "https://cuckoo.sh/vmcloak/AdbeRdr11010_en_US.exe",
            "sha1": "98b2b838e6c4663fefdfd341dfdc596b1eff355c",
        },
    ]

    def run(self):
        self.upload_dependency("C:\\%s" % self.filename)
        self.a.execute(
            "C:\\%s /sAll /msi /norestart /quiet ALLUSERS=1 EULA_ACCEPT=YES" %
            self.filename
        )

        self.a.remove("C:\\%s" % self.filename)

        # add needed registry keys to skip Licence Agreement
        self.a.execute(
            "reg add \"HKEY_LOCAL_MACHINE\\Software\\WOW6432Node\\"
            "Adobe\\Acrobat Reader\\%s.0\\AdobeViewer\" " %
            self.version.split(".")[0]
        )
        self.a.execute(
            "reg add \"HKEY_LOCAL_MACHINE\\Software\\WOW6432Node\\"
            "Adobe\\Acrobat Reader\\%s.0\\AdobeViewer\" "
            "/v EULA /t REG_DWORD /d 1 /f" % self.version.split(".")[0]
        )
        self.a.execute(
            "reg add \"HKEY_LOCAL_MACHINE\\Software\\WOW6432Node\\"
            "Adobe\\Acrobat Reader\\%s.0\\AdobeViewer\" "
            "/v Launched /t REG_DWORD /d 1 /f" % self.version.split(".")[0]
        )

        # man : https://www.adobe.com/devnet-docs/acrobatetk/tools/PrefRef/Windows/
        # we don't care about updates really
        self.a.execute(
            "reg add \"HKEY_CURRENT_USER\\Software\\Adobe\\"
            "Acrobat Reader\\%s.0\\AVGeneral\" "
            "/v bCheckForUpdatesAtStartup /t REG_DWORD /d 0 /f" %
            self.version.split(".")[0]
        )

        # allow URL access
        self.a.execute(
            "reg add \"HKEY_CURRENT_USER\\Software\\Adobe\\"
            "Acrobat Reader\\%s.0\\TrustManager\\cDefaultLaunchURLPerms\" " %
            self.version.split(".")[0]
        )
        self.a.execute(
            "reg add \"HKEY_CURRENT_USER\\Software\\Adobe\\"
            "Acrobat Reader\\%s.0\\TrustManager\\cDefaultLaunchURLPerms\" "
            "/v iURLPerms /t REG_DWORD /d 2 /f" % self.version.split(".")[0]
        )

        # FIXME: really needed ?
        self.a.execute(
            "reg add \"HKEY_CURRENT_USER\\Software\\Adobe\\"
            "Acrobat Reader\\%s.0\\Security\\cDigSig\\cCustomDownload\" "
            "/v bLoadSettingsFromURL /t REG_DWORD /d 0 /f" %
            self.version.split(".")[0]
        )

class Adobe9(AdobePdf, Dependency):
    """Backwards compatibility."""
    name = "adobe9"
    recommended = False



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import logging
import os.path

from vmcloak.abstract import Dependency

log = logging.getLogger(__name__)

class Agent(Dependency):
    name = "agent"

    def init(self):
        self.path = None

    def check(self):
        if not self.path or not os.path.exists(self.path):
            log.error("Please provide the path to the new Agent file.")
            return False

    def run(self):
        self.a.upload("C:\\vmcloak\\agent.py", open(self.path, "rb"))

        # Start the Agent in about 10 seconds from now. That should give us
        # enough time to kill the existing Agent.
        self.a.execute("C:\\Python27\\python.exe C:\\vmcloak\\agent.py "
                       "0.0.0.0 %s -s 10000" % self.a.port, async=True)
        self.a.kill()



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import logging
import os

log = logging.getLogger(__name__)

PATHS = {
    "wget": [
        "/usr/bin/wget",
        "/usr/local/bin/wget",
        "/usr/pkg/bin/wget"
    ],
    "genisoimage": [
        "/usr/bin/genisoimage",
        "/usr/pkg/bin/genisoimage",
        "/usr/bin/mkisofs",
        "/usr/local/bin/mkisofs",
        "/usr/pkg/bin/mkisofs",
    ],
    "vboxmanage": [
        "/usr/bin/VBoxManage",
        "/usr/local/bin/VBoxManage",
    ],
}

INSTALL = {
    "wget": {
        "Linux": "apt-get install wget",
        "NetBSD": "pkgin install wget",
    },
    "genisoimage": {
        "Linux": "apt-get install genisoimage",
        "NetBSD": "pkgin install cdrkit",
    },
    "vboxmanage": {
        "Linux": "apt-get install virtualbox",
    },
}

def get_path(app):
    """Returns the path to an application."""
    for path in PATHS[app]:
        if os.path.isfile(path):
            return path

    log.error("No executable path found for %s!", app)
    log.info("In order to install it, run %r",
             INSTALL[app].get(os.uname()[0], "?"))



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import Dependency

class Python27(Dependency):
    name = "python27"
    exes = [
        {
            "url": "https://cuckoo.sh/vmcloak/python-2.7.6.msi",
            "sha1": "c5d71f339f7edd70ecd54b50e97356191347d355",
        },
    ]

    def run(self):
        """There are no installation procedures for this dependency as it is
        setup by default when installing a new Virtual Machine. In the end,
        Python is required for running both the Agent as well as Cuckoo's
        Analyzer anyway."""



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import Dependency
from vmcloak.misc import import_plugins

plugins = import_plugins(__file__, "vmcloak.dependencies", globals(), Dependency)
names = dict((plugin.name, plugin) for plugin in plugins)



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import hashlib
import importlib
import logging
import os
import shutil
import socket
import stat
import struct
import subprocess
import sys

from ConfigParser import ConfigParser

try:
    import pwd
    HAVE_PWD = True
except ImportError:
    HAVE_PWD = False

log = logging.getLogger(__name__)

def copytreelower(srcdir, dstdir):
    """Copies the source directory as lowercase to the destination directory.

    Lowercase as in, all directory and filenames are translated to lowercase,
    thus emulating Windows case-insensitive filepaths.

    """
    # If the source directory doesn't end with a slash then we have to take
    # this into account when slicing it later on. Namely, when the source
    # directory is "/mnt/winxp" then we slice up to "/mnt/winxp/", thus we
    # require one extra character for the slash. When the source directory is
    # "/mnt/winxp/" then of course we don't an extra character for the slice.
    prefix = len(srcdir) + (not srcdir.endswith("/"))
    for dirpath, dirnames, filenames in os.walk(srcdir):
        for dirname in dirnames:
            os.mkdir(os.path.join(dstdir,
                                  dirpath[prefix:].lower(),
                                  dirname.lower()))

        for fname in filenames:
            path = os.path.join(dirpath, fname)[prefix:]

            # Copy the file.
            shutil.copyfile(os.path.join(srcdir, path),
                            os.path.join(dstdir, path.lower()))

            # Make the file writable.
            os.chmod(os.path.join(dstdir, path.lower()),
                     stat.S_IRUSR | stat.S_IWUSR)

def copytreeinto(srcdir, dstdir):
    """Copy one directory into another directory.

    Unlike shutil.copytree() this function doesn't require the destination
    directory to be non-existing.

    """
    if os.path.isfile(dstdir):
        raise Exception("Cannot create directory if there is already "
                        "a file: %s" % dstdir)

    if not os.path.isdir(dstdir):
        os.mkdir(dstdir)

    for fname in os.listdir(srcdir):
        path_in = os.path.join(srcdir, fname)
        path_out = os.path.join(dstdir, fname)

        if os.path.isfile(path_in):
            shutil.copy(path_in, path_out)
        else:
            copytreeinto(path_in, path_out)

def ini_read(path):
    ret, section = {}, None

    if os.path.exists(path):
        buf = open(path, "rb").read()
    else:
        buf = ""

    # UTF-16 Byte Order Mark ("BOM")
    mode = "utf16" if buf[:2] == "\xff\xfe" else "latin1"
    buf = buf.decode(mode)

    for line in buf.split("\n"):
        line = line.strip()
        if not line or line[0] == ";":
            continue

        if line[0] == "[" and "]" in line:
            section = line[1:line.index("]")]
            ret[section] = []
            continue

        if "=" not in line:
            ret[section].append(line)
        else:
            a, b = line.split("=", 1)
            ret[section].append("%s=%s" % (a.strip(), b.strip()))
    return mode, ret

def ini_write(path, mode, data):
    lines = [""]
    for key in sorted(data.keys()):
        lines.append("[%s]" % key)
        for value in sorted(data[key]):
            lines.append(value)
        lines.append("")
    open(path, "wb").write("\r\n".join(lines).encode(mode))

def ini_add(data, section, value):
    if section not in data:
        data[section] = []

    if value not in data[section]:
        data[section].append(value)

def ini_delete(data, section, value):
    if section not in data:
        return

    for idx, row in enumerate(data[section]):
        if row == value:
            del data[section][idx]

def ini_merge(data, ini2, overwrite=True):
    mode, data2 = ini_read(ini2)
    for section in data2:
        for value in data2[section]:
            if section not in data:
                data[section] = [value]
                continue

            off = value.find("=")
            # If this line is not a key = value entry, then
            # we take the entire line.
            if off < 0:
                off = len(value)

            for idx, row in enumerate(data[section]):
                if len(row) > off and row[:off] == value[:off]:
                    if overwrite:
                        data[section][idx] = value
                    break
            else:
                data[section].append(value)

def ini_read_dict(path):
    c = ConfigParser()
    c.read(path)

    ret = {}
    for section in c.sections():
        ret[section] = dict(c.items(section))
    return ret

def sha1_file(path):
    """Calculate the sha1 hash of a file."""
    h = hashlib.sha1()
    f = open(path, "rb")

    while True:
        buf = f.read(1024*1024)
        if not buf:
            break

        h.update(buf)

    return h.hexdigest()

def register_cuckoo(hostonly_ip, tags, vmname, cuckoo_dirpath, rdp_port=None):
    log.debug("Registering the Virtual Machine with Cuckoo.")

    # Legacy Cuckoo setup.
    machine_py = os.path.join(cuckoo_dirpath, "utils", "machine.py")
    if os.path.exists(machine_py):
        try:
            args = [
                machine_py, "--add",
                "--ip", hostonly_ip,
                "--platform", "windows",
                "--tags", tags or "",
                "--snapshot", "vmcloak",
                vmname,
            ]

            if rdp_port:
                args += ["--rdp_port", "%s" % rdp_port]

            subprocess.check_call(args, cwd=cuckoo_dirpath)
            return True
        except OSError as e:
            log.error("Is $CUCKOO/utils/machine.py executable? -> %s", e)
            return False
        except subprocess.CalledProcessError as e:
            log.error("Error registering the VM: %s.", e)
            return False
        return

    # Cuckoo Package setup.
    try:
        args = [
            "cuckoo",
            "--cwd", cuckoo_dirpath,
            "machine",
            "--add", vmname, hostonly_ip,
            "--platform", "windows",
            "--tags", tags or "",
            "--snapshot", "vmcloak",
        ]
        subprocess.check_call(args)
    except subprocess.CalledProcessError as e:
        log.error("Error registering VM: %s", e)
        return False

def wait_for_host(ipaddr, port):
    # Wait for the Agent to come up with a timeout of 1 second.
    while True:
        try:
            log.debug("Waiting for host %s", ipaddr)
            socket.create_connection((ipaddr, port), 1).close()
            break
        except socket.error:
            pass

def drop_privileges(user):
    if not HAVE_PWD:
        sys.exit(
            "This Operating System does not support the pwd module, please "
            "don't use privilege dropping."
        )

    try:
        user = pwd.getpwnam(user)
        os.setgroups((user.pw_gid,))
        os.setgid(user.pw_gid)
        os.setuid(user.pw_uid)
        os.environ["HOME"] = user.pw_dir
    except KeyError:
        sys.exit("Invalid user specified to drop privileges to: %s" % user)
    except OSError as e:
        sys.exit("Failed to drop privileges: %s" % e)

def import_plugins(dirpath, module_prefix, namespace, class_):
    """Import plugins of type `class` located at `dirpath` into the
    `namespace` that starts with `module_prefix`. If `dirpath` represents a
    filepath then it is converted into its containing directory."""
    if os.path.isfile(dirpath):
        dirpath = os.path.dirname(dirpath)

    for fname in os.listdir(dirpath):
        if fname.endswith(".py") and not fname.startswith("__init__"):
            module_name, _ = os.path.splitext(fname)
            importlib.import_module("%s.%s" % (module_prefix, module_name))

    plugins = []
    for subclass in class_.__subclasses__():
        namespace[subclass.__name__] = subclass
        plugins.append(subclass)
    return plugins

def ipaddr_increase(ipaddr):
    """Increases the IP address."""
    addr = struct.unpack(">I", socket.inet_aton(ipaddr))[0]
    return socket.inet_ntoa(struct.pack(">I", addr + 1))



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import logging
import os
import subprocess
import time

from vmcloak.abstract import Machinery
from vmcloak.data.config import VBOX_CONFIG
from vmcloak.exceptions import CommandError
from vmcloak.paths import get_path
from vmcloak.rand import random_mac
from vmcloak.repository import vms_path

log = logging.getLogger(__name__)

class VirtualBox(Machinery):
    FIELDS = VBOX_CONFIG

    def __init__(self, *args, **kwargs):
        Machinery.__init__(self, *args, **kwargs)
        self.vboxmanage = get_path("vboxmanage")

    def _call(self, *args, **kwargs):
        cmd = [self.vboxmanage] + list(args)

        for k, v in kwargs.items():
            if v is None or v is True:
                cmd += ["--" + k]
            else:
                cmd += ["--" + k.rstrip("_"), str(v)]

        try:
            log.debug("Running command: %s", cmd)
            ret = subprocess.check_output(cmd)
        except Exception as e:
            log.error("[-] Error running command: %s", e)
            raise CommandError

        return ret.strip()

    def vminfo(self, element=None):
        ret = {}
        lines = self._call("showvminfo", self.name, machinereadable=True)
        for line in lines.split("\n"):
            key, value = line.split("=", 1)

            if value.startswith('"') and value.endswith('"'):
                value = value[1:-1]
            elif value.isdigit():
                value = int(value)

            if key.startswith('"') and key.endswith('"'):
                key = key[1:-1]

            ret[key] = value
        return ret if element is None else ret.get(element)

    def wait_for_state(self, shutdown=False):
        while True:
            try:
                status = self.vminfo("VMState")
                if shutdown and status == "poweroff":
                    break
            except CommandError:
                pass

            time.sleep(1)

    def create_vm(self):
        return self._call("createvm", name=self.name,
                          basefolder=vms_path, register=True)

    def delete_vm(self):
        self._call("unregistervm", self.name, delete=True)

    def ramsize(self, ramsize):
        return self._call("modifyvm", self.name, memory=ramsize)

    def vramsize(self, vramsize):
        return self._call("modifyvm", self.name, vram=vramsize)

    def os_type(self, osversion):
        operating_systems = {
            "winxp": "WindowsXP",
            "win7x86": "Windows7",
            "win7x64": "Windows7_64",
            "win81x86": "Windows81",
            "win81x64": "Windows81_64",
            "win10x86": "Windows10",
            "win10x64": "Windows10_64",
        }
        return self._call("modifyvm", self.name,
                          ostype=operating_systems[osversion])

    def create_hd(self, hdd_path, fsize=256*1024):
        self._call("createhd", filename=hdd_path, size=fsize)
        self._call("storagectl", self.name, name="IDE", add="ide")
        self._call("storageattach", self.name, storagectl="IDE",
                   type_="hdd", device=0, port=0, medium=hdd_path)

    def attach_hd(self, hdd_path, multi=False):
        # When a harddisk is not attached to a Virtual Machine it will quickly
        # be forgotten. This seems to be within a couple of seconds. When this
        # happens, its "type" (multiattach in our case) is also forgotten,
        # resulting in issues when cloning. Therefore we quickly set its state
        # before attaching it to a Virtual Machine, hoping this approach
        # is "good enough".
        self._call("storagectl", self.name, name="IDE", add="ide")
        if multi:
            self._call("modifyhd", hdd_path, type_="multiattach")
        else:
            self._call("modifyhd", hdd_path, type_="normal")
        self._call("storageattach", self.name, storagectl="IDE",
                   type_="hdd", device=0, port=0, medium=hdd_path)

    def compact_hd(self, hdd_path):
        # We first make the HDD "more" compact - this should be basically
        # defragmenting it.
        self._call("modifyhd", hdd_path, compact=True)

    def clone_hd(self, hdd_inpath, hdd_outpath):
        self._call("clonehd", hdd_inpath, hdd_outpath)

    def remove_hd(self):
        time.sleep(1)
        self._call("storagectl", self.name, portcount=0,
                   name="IDE", remove=True)

    def cpus(self, count):
        self._call("modifyvm", self.name, cpus=count, ioapic="on")

    def attach_iso(self, iso_path):
        """Mount an ISO to the Virtual Machine."""
        self._call("storageattach", self.name, storagectl="IDE",
                   type_="dvddrive", port=1, device=0, medium=iso_path)

    def detach_iso(self):
        time.sleep(1)
        self._call("storageattach", self.name, storagectl="IDE",
                   type_="dvddrive", port=1, device=0, medium="emptydrive")

    def set_field(self, key, value):
        return self._call("setextradata", self.name, key, value)

    def modify_mac(self, macaddr=None, index=1):
        if macaddr is None:
            macaddr = random_mac()

        # VBoxManage prefers MAC addresses without colons.
        vbox_mac = macaddr.replace(":", "")

        mac = {"macaddress%d" % index: vbox_mac}
        self._call("modifyvm", self.name, **mac)
        return macaddr

    def hostonly(self, nictype, macaddr=None, adapter=None):
        index = self.network_index() + 1
        if not adapter:
            if os.name == "posix":
                adapter = "vboxnet0"
            else:
                adapter = "VirtualBox Host-Only Ethernet Adapter"

        # Ensure our hostonly interface is actually up and running.
        if adapter not in self._call("list", "hostonlyifs"):
            log.error("Have you configured %s?", adapter)
            log.info("Please refer to the documentation to configure it.")
            return False

        nic = {
            "nic%d" % index: "hostonly",
            "nictype%d" % index: nictype,
            "nicpromisc%d" % index: "allow-all",
            "hostonlyadapter%d" % index: adapter,
            "cableconnected%d" % index: "on",
        }
        self._call("modifyvm", self.name, **nic)
        return self.modify_mac(macaddr, index)

    def nat(self, nictype, macaddr=None):
        index = self.network_index() + 1

        nic = {
            "nic%d" % index: "nat",
            "nictype%d" % index: nictype,
            "nicpromisc%d" % index: "allow-all",
        }
        self._call("modifyvm", self.name, **nic)
        return self.modify_mac(macaddr, index)

    def hwvirt(self, enable=True):
        """Enable or disable the usage of Hardware Virtualization."""
        self._call("modifyvm", self.name, hwvirtex="on" if enable else "off")

    def start_vm(self, visible=False):
        return self._call("startvm", self.name,
                          type_="gui" if visible else "headless")

    def snapshot(self, label, description=""):
        return self._call("snapshot", self.name, "take", label,
                          description=description, live=True)

    def restore_snapshot(self, label=None):
        if label:
            return self._call("snapshot", self.name, "restore", label)
        else:
            return self._call("snapshot", self.name, "restorecurrent")

    def delete_snapshot(self, label):
        return self._call("snapshot", self.name, "delete", label)

    def stopvm(self):
        return self._call("controlvm", self.name, "poweroff")

    def list_settings(self):
        return self._call("getextradata", self.name, "enumerate")

    def mouse(self, type):
        return self._call("modifyvm", self.name, mouse=type)

    def vrde(self, port, password):
        return self._call("modifyvm", self.name, vrde="on", vrdeport=port,
                          vrdeproperty="VNCPassword=%s" % password)

    def export(self, filepath):
        return self._call(
            "export", self.name, "--output", filepath, "--vsys", "0",
            product="VMCloak",
            producturl="http://vmcloak.org/",
            vendor="Cuckoo Sandbox",
            vendorurl="http://cuckoosandbox.org/",
            description="Cuckoo Sandbox Virtual Machine created by VMCloak",
        )



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import os.path

VMCLOAK_ROOT = os.path.abspath(os.path.dirname(__file__))
VMCLOAK_VM_MODES = ["virtualbox", "iso"]



#!/usr/bin/env python
# Copyright (C) 2014-2017 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from setuptools import setup


setup(
    name="VMCloak",
    version="0.4.3a1",
    author="Jurriaan Bremer",
    author_email="jbr@cuckoo.sh",
    url="http://vmcloak.org/",
    packages=[
        "vmcloak",
        "vmcloak.data",
        "vmcloak.data.bootstrap",
        "vmcloak.data.hwconf",
    ],
    scripts=[
        "bin/vmcloak-gethwconf",
        "bin/vmcloak-iptables",
        "bin/vmcloak-killvbox",
        "bin/vmcloak-removevms",
        "bin/vmcloak-vboxnet0",
    ],
    entry_points={
        "console_scripts": [
            "vmcloak = vmcloak.main:main",
        ],
    },
    license="GPLv3",
    description="Automated Virtual Machine Generation and Cloaking "
                "for Cuckoo Sandbox.",
    include_package_data=True,
    package_data={
        "vmcloak.data": ["*.*"],
        "vmcloak.data.bootstrap": ["*.*"],
        "vmcloak.data.winxp": ["*.*"],
        "vmcloak.data.win7": ["*.*"],
    },
    install_requires=[
        "click==6.6",
        "jinja2==2.8",
        "pyyaml==3.12",
        "sphinx==1.5a1",
        "sqlalchemy==1.0.8",
    ],
    extras_require={
        ":sys_platform == 'win32'": [
            "requests==2.7.0",
        ],
        ":sys_platform == 'darwin'": [
            "requests==2.7.0",
        ],
        ":sys_platform == 'linux2'": [
            "requests[security]==2.7.0",
        ],
    },
)



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import WindowsAutounattended

class Windows10(WindowsAutounattended):
    name = "win10"
    service_pack = 2
    interface = "Ethernet"

    # List of preferences when multiple Windows 10 types are available.
    preference = "pro", "enterprise", "home"

    dummy_serial_key = "W269N-WFGWX-YVC9B-4J6C9-T83GX"

class Windows10x64(Windows10):
    arch = "amd64"
    mount = "/mnt/win10x64", "/mnt/win10"

class Windows10x86(Windows10):
    arch = "x86"
    mount = "/mnt/win10x86", "/mnt/win10"



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import logging
import os
import tempfile

from vmcloak.abstract import OperatingSystem
from vmcloak.misc import ini_merge, ini_read, ini_write
from vmcloak.rand import random_string
from vmcloak.verify import valid_serial_key

log = logging.getLogger(__name__)

class WindowsXP(OperatingSystem):
    name = "winxp"
    service_pack = 3
    mount = "/mnt/winxp"
    nictype = "Am79C973"
    osdir = os.path.join("$oem$", "$1")
    genisoargs = [
        "-no-emul-boot", "-boot-load-seg", "1984", "-boot-load-size", "4",
        "-iso-level", "2", "-J", "-l", "-D", "-N", "-joliet-long",
        "-relaxed-filenames",
    ]
    interface = "Local Area Connection"

    def _winnt_sif(self):
        values = {
            "PRODUCTKEY": self.serial_key,
            "COMPUTERNAME": random_string(8, 16),
            "FULLNAME": "%s %s" % (random_string(4, 8), random_string(4, 10)),
            "ORGANIZATION": "",
            "WORKGROUP": random_string(4, 8),
            # "KBLAYOUT": s.keyboard_layout,
            "KBLAYOUT": "US",
        }

        buf = open(os.path.join(self.path, "winnt.sif"), "rb").read()
        for key, value in values.items():
            buf = buf.replace("@%s@" % key, value)

        fd, winntsif = tempfile.mkstemp(suffix=".sif", dir=self.tempdir)
        os.write(fd, buf)
        os.close(fd)

        return winntsif

    def isofiles(self, outdir, tmp_dir=None):
        dst_winnt = os.path.join(outdir, "i386", "winnt.sif")

        winnt_sif = self._winnt_sif()

        # Merge the existing winnt.sif with our changes.
        mode, winnt = ini_read(dst_winnt)
        ini_merge(winnt, winnt_sif, overwrite=True)

        os.remove(winnt_sif)

        # There are a couple of optional values that should be set if they have
        # not already been set.
        winnt_opt_sif = os.path.join(self.path, "winnt-opt.sif")
        ini_merge(winnt, winnt_opt_sif, overwrite=False)

        ini_write(dst_winnt, mode, winnt)

    def set_serial_key(self, serial_key):
        if not serial_key:
            log.error("No serial key has been provided - please do so!")
            return False

        if not valid_serial_key(serial_key):
            log.error("The provided serial key has an incorrect encoding.")
            log.info("Example encoding, AAAAA-BBBBB-CCCCC-DDDDD-EEEEE.")
            return False

        self.serial_key = serial_key
        return True



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

class CommandError(Exception):
    pass

class DependencyError(Exception):
    pass

class SwarmError(Exception):
    pass



# Copyright (C) 2014-2016 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from vmcloak.abstract import Dependency

class IE11(Dependency):
    name = "ie11"
    default = "11"
    depends = [
        "kb:2670838", "kb:2639308", "kb:2533623", "kb:2731771", "kb:2729094",
        "kb:2786081", "kb:2882822", "kb:2888049", "kb:2834140",
    ]
    exes = [
        {
            "version": "11",
            "target": "win7x64",
            "url": "https://cuckoo.sh/vmcloak/IE11-Windows6.1-x64-en-us.exe",
            "sha1": "ddec9ddc256ffa7d97831af148f6cc45130c6857",
        },
        {
            "version": "11",
            "target": "win7x86",
            "url": "https://cuckoo.sh/vmcloak/IE11-Windows6.1-x86-en-us.exe",
            "sha1": "fefdcdde83725e393d59f89bb5855686824d474e",
        },
    ]

    def run(self):
        self.upload_dependency("C:\\setup.exe")
        self.a.execute("C:\\setup.exe /quiet /norestart /update-no")
        self.a.remove("C:\\setup.exe")
